@@ -146,6 +146,8 @@ def test_date_functions(test_environment):
         date_trunc(order_timestamp, hour) -> order_hour_trunc,
         date_trunc(order_timestamp, minute) -> order_minute_trunc,
         date_trunc(order_timestamp, second) -> order_second_trunc,
+        date_trunc(order_timestamp, quarter) -> order_quarter_trunc,
+        date_trunc(order_timestamp, week) -> order_week_trunc,
         date_part(order_timestamp, month) -> order_month_part,
         date_part(order_timestamp, day) -> order_day_part,
         date_part(order_timestamp, year) -> order_year_part,
@@ -155,6 +157,8 @@ def test_date_functions(test_environment):
         date_part(order_timestamp, quarter) -> order_quarter_part,
         date_part(order_timestamp, week) -> order_week_part,
         date_part(order_timestamp, day_of_week) -> order_day_of_week_part,
+        month_name(order_timestamp) -> order_month_name,
+        day_name(order_timestamp) -> order_day_name
     ;
     
     
@@ -163,7 +167,28 @@ def test_date_functions(test_environment):
     select: SelectStatement = parsed[-1]
 
     for dialect in TEST_DIALECTS:
+        engine = None
+        if isinstance(dialect, DuckDBDialect):
+            engine = Dialects.DUCK_DB.default_executor(environment=test_environment)
         dialect.compile_statement(process_query(test_environment, select))
+        if engine:
+            engine.execute_raw_sql(
+                """CREATE TABLE tblRevenue AS
+SELECT 
+    100.50 as revenue,
+    1 as order_id,
+    101 as product_id,
+    TIMESTAMP '2024-01-15 10:30:00' as order_timestamp
+UNION ALL
+SELECT 
+    250.75 as revenue,
+    2 as order_id,
+    102 as product_id,
+    TIMESTAMP '2024-01-16 14:45:30' as order_timestamp;"""
+            )
+            results = engine.execute_query(select)
+            assert results
+            assert results.fetchall()
 
 
 def test_bad_cast(test_environment):
@@ -233,6 +258,7 @@ def test_math_functions(test_environment):
     property order_id.rounded_default <- round(revenue + 2.01);
     property order_id.floor <- floor(revenue + 2.01);
     property order_id.ceil <- ceil(revenue + 2.01);
+    property order_id.square <- revenue **2;
     constant random <- random(1);
     select
         order_id,
@@ -246,6 +272,7 @@ def test_math_functions(test_environment):
         floor,
         ceil,
         random,
+        square,
     ;"""
     env, parsed = parse(declarations, environment=test_environment)
     select: SelectStatement = parsed[-1]
@@ -283,16 +310,43 @@ def test_string_functions(test_environment):
     property regex_replace <- regexp_replace(category_name, 'a', 'b');
 
     select
+        category_id,
         test_name,
         upper_name,
         lower_name,
         substring_name,
         strpos_name,
+        like_name,
+        like_alt,
+        regex_contains,
+        regex_substring,
+        regex_replace,
+        hash(category_name, md5) -> hash_md5,
+        hash(category_name, sha1) -> hash_sha1,
+        hash(category_name, sha256) -> hash_sha256,
+        # hash(category_name, sha512) -> hash_sha512
     ;"""
     env, parsed = parse(declarations, environment=test_environment)
     select: SelectStatement = parsed[-1]
     for dialect in TEST_DIALECTS:
+        engine = None
+        if isinstance(dialect, DuckDBDialect):
+            engine = Dialects.DUCK_DB.default_executor(environment=test_environment)
         dialect.compile_statement(process_query(test_environment, select))
+        if engine:
+            engine.execute_raw_sql(
+                """CREATE TABLE tblCategory AS
+SELECT 
+    2 category_id,
+    'category_a' as category_name
+UNION ALL
+SELECT 
+    3 category_id,
+    'category_b' as category_name;"""
+            )
+            results = engine.execute_query(select)
+            assert results
+            assert results.fetchall()
 
 
 def test_case_function(test_environment):
