@@ -1,7 +1,9 @@
 import random
+from contextlib import contextmanager
 from dataclasses import dataclass, field
 from enum import Enum
 from logging import getLogger
+from typing import Any
 
 logger = getLogger("trilogy")
 
@@ -29,6 +31,7 @@ class Optimizations:
     constant_inlining: bool = True
     constant_inline_cutoff: int = 10
     direct_return: bool = True
+    hide_unused_concepts: bool = True
 
 
 @dataclass
@@ -37,9 +40,9 @@ class Comments:
 
     show: bool = False
     basic: bool = True
-    joins: bool = True
-    nullable: bool = True
-    partial: bool = True
+    joins: bool = False
+    nullable: bool = False
+    partial: bool = False
     source_map: bool = False
 
 
@@ -50,6 +53,32 @@ class Rendering:
     parameters: bool = True
     concise: bool = False
 
+    @contextmanager
+    def temporary(self, **kwargs: Any):
+        """
+        Context manager to temporarily set attributes and revert them afterwards.
+
+        Usage:
+            r = Rendering()
+            with r.temporary(parameters=False, concise=True):
+                # parameters is False, concise is True here
+                do_something()
+            # parameters and concise are back to their original values
+        """
+        # Store original values
+        original_values = {key: getattr(self, key) for key in kwargs}
+
+        # Set new values
+        for key, value in kwargs.items():
+            setattr(self, key, value)
+
+        try:
+            yield self
+        finally:
+            # Restore original values
+            for key, value in original_values.items():
+                setattr(self, key, value)
+
 
 @dataclass
 class Parsing:
