@@ -169,6 +169,7 @@ class FunctionType(Enum):
     ARRAY_SORT = "array_sort"
     ARRAY_TRANSFORM = "array_transform"
     ARRAY_TO_STRING = "array_to_string"
+    ARRAY_FILTER = "array_filter"
 
     # MAP
     MAP_KEYS = "map_keys"
@@ -178,7 +179,7 @@ class FunctionType(Enum):
     SPLIT = "split"
     LENGTH = "len"
 
-    # Math
+    # Mathas
     DIVIDE = "divide"
     MULTIPLY = "multiply"
     ADD = "add"
@@ -191,6 +192,7 @@ class FunctionType(Enum):
     FLOOR = "floor"
     CEIL = "ceil"
     LOG = "log"
+    POWER = "power"
 
     # Aggregates
     ## group is not a real aggregate - it just means group by this + some other set of fields
@@ -204,6 +206,7 @@ class FunctionType(Enum):
     MIN = "min"
     AVG = "avg"
     ARRAY_AGG = "array_agg"
+    ANY = "any"
 
     # String
     LIKE = "like"
@@ -215,6 +218,7 @@ class FunctionType(Enum):
     CONTAINS = "contains"
     TRIM = "trim"
     REPLACE = "replace"
+    HASH = "hash"
 
     # STRING REGEX
     REGEXP_CONTAINS = "regexp_contains"
@@ -236,12 +240,15 @@ class FunctionType(Enum):
     MONTH = "month"
     QUARTER = "quarter"
     YEAR = "year"
+    MONTH_NAME = "month_name"
+    DAY_NAME = "day_name"
 
     DATE_PART = "date_part"
     DATE_TRUNCATE = "date_truncate"
     DATE_ADD = "date_add"
     DATE_SUB = "date_sub"
     DATE_DIFF = "date_diff"
+    DATE_SPINE = "date_spine"
 
     # UNIX
     UNIX_TO_TIMESTAMP = "unix_to_timestamp"
@@ -261,6 +268,7 @@ class FunctionClass(Enum):
         FunctionType.ARRAY_AGG,
         FunctionType.COUNT,
         FunctionType.COUNT_DISTINCT,
+        FunctionType.ANY,
     ]
     SINGLE_ROW = [
         FunctionType.CONSTANT,
@@ -268,7 +276,7 @@ class FunctionClass(Enum):
         FunctionType.CURRENT_DATETIME,
     ]
 
-    ONE_TO_MANY = [FunctionType.UNNEST]
+    ONE_TO_MANY = [FunctionType.UNNEST, FunctionType.DATE_SPINE]
 
     RECURSIVE = [FunctionType.RECURSE_EDGE]
 
