@@ -20,7 +20,8 @@ def add_concept(
     if node_name in seen:
         return
     seen.add(node_name)
-    g.add_node(concept)
+    g.concepts[node_name] = concept
+    g.add_node(node_name)
     if concept.concept_arguments:
         for source in concept.concept_arguments:
             if not isinstance(source, BuildConcept):
@@ -28,9 +29,10 @@ def add_concept(
                     f"Invalid non-build concept {source} passed into graph generation from {concept}"
                 )
             generic = get_default_grain_concept(source, default_concept_graph)
+            generic_node = concept_to_node(generic)
             add_concept(generic, g, concept_mapping, default_concept_graph, seen)
 
-            g.add_edge(generic, node_name)
+            g.add_edge(generic_node, node_name, fast=True)
     for ps_address in concept.pseudonyms:
         if ps_address not in concept_mapping:
             raise SyntaxError(f"Concept {concept} has invalid pseudonym {ps_address}")
@@ -44,8 +46,8 @@ def add_concept(
             continue
         if pseudonym_node.split("@")[0] == node_name.split("@")[0]:
             continue
-        g.add_edge(pseudonym_node, node_name)
-        g.add_edge(node_name, pseudonym_node)
+        g.add_edge(pseudonym_node, node_name, fast=True)
+        g.add_edge(node_name, pseudonym_node, fast=True)
         g.pseudonyms.add((pseudonym_node, node_name))
         g.pseudonyms.add((node_name, pseudonym_node))
         add_concept(pseudonym, g, concept_mapping, default_concept_graph, seen)
@@ -82,20 +84,27 @@ def generate_adhoc_graph(
 
     for dataset in datasources:
         node = datasource_to_node(dataset)
-        g.add_node(dataset, type="datasource", datasource=dataset)
+        g.add_datasource_node(node, dataset)
         for concept in dataset.concepts:
+            cnode = concept_to_node(concept)
+            g.concepts[cnode] = concept
+            g.add_node(cnode)
             if restrict_to_listed:
-                if concept_to_node(concept) not in g.nodes:
+                if cnode not in g.nodes:
                     continue
-            g.add_edge(node, concept)
-            g.add_edge(concept, node)
+            g.add_edge(node, cnode, fast=True)
+            g.add_edge(cnode, node, fast=True)
             # if there is a key on a table at a different grain
             # add an FK edge to the canonical source, if it exists
             # for example, order ID on order product table
             default = get_default_grain_concept(concept, default_concept_graph)
+
             if concept != default:
-                g.add_edge(concept, default)
-                g.add_edge(default, concept)
+                dcnode = concept_to_node(default)
+                g.concepts[dcnode] = default
+                g.add_node(dcnode)
+                g.add_edge(cnode, dcnode, fast=True)
+                g.add_edge(dcnode, cnode, fast=True)
     return g
 
 
