@@ -19,6 +19,7 @@ from trilogy.core.models.author import (
     Concept,
     ConceptRef,
     Conditional,
+    CustomType,
     Function,
     Parenthetical,
     UndefinedConcept,
@@ -40,6 +41,11 @@ from trilogy.core.models.environment import Environment
 GENERIC_ARGS = Concept | ConceptRef | Function | str | int | float | date | datetime
 
 
+CUSTOM_PLACEHOLDER = CustomType(
+    name="__placeholder__", type=DataType.UNKNOWN, drop_on=[], add_on=[]
+)
+
+
 @dataclass
 class FunctionConfig:
     arg_count: int = 1
@@ -99,7 +105,18 @@ def get_attr_datatype(
 
 def get_cast_output_type(
     args: list[Any],
-) -> DataType:
+) -> DataType | TraitDataType:
+    base = arg_to_datatype(args[0])
+    if isinstance(base, TraitDataType):
+        traits = base.traits
+    else:
+        traits = []
+    if isinstance(args[1], TraitDataType):
+        return TraitDataType(
+            type=args[1].type, traits=list(set(traits + args[1].traits))
+        )
+    elif traits:
+        return TraitDataType(type=args[1], traits=traits)
     return args[1]
 
 
@@ -175,6 +192,10 @@ def get_date_trunc_output(
         return DataType.DATETIME
     elif target == DatePart.SECOND:
         return DataType.DATETIME
+    elif target == DatePart.WEEK:
+        return DataType.DATE
+    elif target == DatePart.QUARTER:
+        return DataType.DATE
     else:
         raise InvalidSyntaxException(f"Date truncation not supported for {target}")
 
@@ -208,6 +229,14 @@ FUNCTION_REGISTRY: dict[FunctionType, FunctionConfig] = {
         output_type_function=get_unnest_output_type,
         arg_count=1,
     ),
+    FunctionType.DATE_SPINE: FunctionConfig(
+        valid_inputs={
+            DataType.DATE,
+        },
+        output_purpose=Purpose.KEY,
+        output_type=DataType.DATE,
+        arg_count=2,
+    ),
     FunctionType.RECURSE_EDGE: FunctionConfig(
         arg_count=2,
     ),
@@ -314,6 +343,18 @@ FUNCTION_REGISTRY: dict[FunctionType, FunctionConfig] = {
         output_type_function=get_transform_output_type,
         arg_count=3,
     ),
+    FunctionType.ARRAY_FILTER: FunctionConfig(
+        valid_inputs=[
+            {
+                DataType.ARRAY,
+            },
+            {*DataType},
+            {*DataType},
+        ],
+        output_purpose=Purpose.PROPERTY,
+        output_type_function=get_transform_output_type,
+        arg_count=3,
+    ),
     FunctionType.ARRAY_TO_STRING: FunctionConfig(
         valid_inputs={
             DataType.ARRAY,
@@ -640,6 +681,17 @@ FUNCTION_REGISTRY: dict[FunctionType, FunctionConfig] = {
         output_type=TraitDataType(type=DataType.INTEGER, traits=["day"]),
         arg_count=1,
     ),
+    FunctionType.DAY_NAME: FunctionConfig(
+        valid_inputs={
+            DataType.DATE,
+            DataType.TIMESTAMP,
+            DataType.DATETIME,
+            # DataType.STRING,
+        },
+        output_purpose=Purpose.PROPERTY,
+        output_type=TraitDataType(type=DataType.STRING, traits=["day_name"]),
+        arg_count=1,
+    ),
     FunctionType.WEEK: FunctionConfig(
         valid_inputs={
             DataType.DATE,
@@ -662,6 +714,17 @@ FUNCTION_REGISTRY: dict[FunctionType, FunctionConfig] = {
         output_type=TraitDataType(type=DataType.INTEGER, traits=["month"]),
         arg_count=1,
     ),
+    FunctionType.MONTH_NAME: FunctionConfig(
+        valid_inputs={
+            DataType.DATE,
+            DataType.TIMESTAMP,
+            DataType.DATETIME,
+            # DataType.STRING,
+        },
+        output_purpose=Purpose.PROPERTY,
+        output_type=TraitDataType(type=DataType.STRING, traits=["month_name"]),
+        arg_count=1,
+    ),
     FunctionType.QUARTER: FunctionConfig(
         valid_inputs={
             DataType.DATE,
@@ -725,6 +788,16 @@ FUNCTION_REGISTRY: dict[FunctionType, FunctionConfig] = {
         output_purpose=Purpose.PROPERTY,
         arg_count=InfiniteFunctionArgs,
     ),
+    FunctionType.POWER: FunctionConfig(
+        valid_inputs={
+            DataType.INTEGER,
+            DataType.FLOAT,
+            DataType.NUMBER,
+            DataType.NUMERIC,
+        },
+        output_purpose=Purpose.PROPERTY,
+        arg_count=2,
+    ),
     FunctionType.DIVIDE: FunctionConfig(
         valid_inputs={
             DataType.INTEGER,
@@ -861,6 +934,11 @@ FUNCTION_REGISTRY: dict[FunctionType, FunctionConfig] = {
         ),
         arg_count=1,
     ),
+    FunctionType.ANY: FunctionConfig(
+        valid_inputs={*DataType},
+        output_purpose=Purpose.PROPERTY,
+        arg_count=1,
+    ),
     FunctionType.AVG: FunctionConfig(
         valid_inputs={
             DataType.INTEGER,
@@ -877,6 +955,14 @@ FUNCTION_REGISTRY: dict[FunctionType, FunctionConfig] = {
         output_type=DataType.TIMESTAMP,
         arg_count=1,
     ),
+    FunctionType.HASH: FunctionConfig(
+        valid_inputs={
+            DataType.STRING,
+        },
+        output_purpose=Purpose.PROPERTY,
+        output_type=DataType.STRING,
+        arg_count=2,
+    ),
 }
 
 EXCLUDED_FUNCTIONS = {
@@ -936,6 +1022,19 @@ class FunctionFactory:
             final_output_type = base_output_type
         else:
             raise SyntaxError(f"Could not determine output type for {operator}")
+        if isinstance(final_output_type, TraitDataType) and self.environment:
+            final_output_type = TraitDataType(
+                type=final_output_type.type,
+                traits=[
+                    x
+                    for x in final_output_type.traits
+                    if operator
+                    not in self.environment.data_types.get(
+                        x, CUSTOM_PLACEHOLDER
+                    ).drop_on
+                ],
+            )
+
         if not output_purpose:
             if operator in FunctionClass.AGGREGATE_FUNCTIONS.value:
                 output_purpose = Purpose.METRIC
