@@ -162,14 +162,13 @@ class ConceptRef(Addressable, Namespaced, DataTyped, Mergeable, BaseModel):
         )
 
     def with_reference_replacement(self, source: str, target: Expr | ArgBinding):
-        if self.address == source:
-            return target
-
         # a reference might be to an attribute of a struct that is bound late
         # if the replacement is a parent in the access path; replace reference
         # with an attribute access call
-        candidates = [f"local.{self.address}", self.address]
+        candidates = [f"{DEFAULT_NAMESPACE}.{self.address}", self.address]
         for candidate in candidates:
+            if candidate == source:
+                return target
             if not candidate.startswith(f"{source}."):
                 continue
             attribute = self.address.rsplit(".", 1)[1]
@@ -461,6 +460,11 @@ class WhereClause(Mergeable, ConceptArgs, Namespaced, BaseModel):
             conditional=self.conditional.with_namespace(namespace)
         )
 
+    def with_reference_replacement(self, source, target):
+        return self.__class__.model_construct(
+            conditional=self.conditional.with_reference_replacement(source, target)
+        )
+
 
 class HavingClause(WhereClause):
     pass
@@ -860,6 +864,7 @@ class Concept(Addressable, DataTyped, ConceptArgs, Mergeable, Namespaced, BaseMo
             AggregateWrapper,
             RowsetItem,
             MultiSelectLineage,
+            Comparison,
         ]
     ] = None
     namespace: str = Field(default=DEFAULT_NAMESPACE, validate_default=True)
@@ -1074,6 +1079,7 @@ class Concept(Addressable, DataTyped, ConceptArgs, Mergeable, Namespaced, BaseMo
         | AggregateWrapper
         | RowsetItem
         | MultiSelectLineage
+        | Comparison
         | None,
         Grain,
         set[str] | None,
@@ -1173,6 +1179,7 @@ class Concept(Addressable, DataTyped, ConceptArgs, Mergeable, Namespaced, BaseMo
                     AggregateWrapper,
                     RowsetItem,
                     MultiSelectLineage,
+                    Comparison,
                 ],
                 output: List[ConceptRef],
             ):
@@ -1199,6 +1206,7 @@ class Concept(Addressable, DataTyped, ConceptArgs, Mergeable, Namespaced, BaseMo
     def calculate_derivation(self, lineage, purpose: Purpose) -> Derivation:
         from trilogy.core.models.build import (
             BuildAggregateWrapper,
+            BuildComparison,
             BuildFilterItem,
             BuildFunction,
             BuildMultiSelectLineage,
@@ -1212,8 +1220,12 @@ class Concept(Addressable, DataTyped, ConceptArgs, Mergeable, Namespaced, BaseMo
             return Derivation.FILTER
         elif lineage and isinstance(lineage, (BuildAggregateWrapper, AggregateWrapper)):
             return Derivation.AGGREGATE
+        # elif lineage and isinstance(lineage, (BuildParenthetical, Parenthetical)):
+        #     return Derivation.PARENTHETICAL
         elif lineage and isinstance(lineage, (BuildRowsetItem, RowsetItem)):
             return Derivation.ROWSET
+        elif lineage and isinstance(lineage, BuildComparison):
+            return Derivation.BASIC
         elif lineage and isinstance(
             lineage, (BuildMultiSelectLineage, MultiSelectLineage)
         ):
@@ -1227,7 +1239,7 @@ class Concept(Addressable, DataTyped, ConceptArgs, Mergeable, Namespaced, BaseMo
         elif (
             lineage
             and isinstance(lineage, (BuildFunction, Function))
-            and lineage.operator == FunctionType.UNNEST
+            and lineage.operator in FunctionClass.ONE_TO_MANY.value
         ):
             return Derivation.UNNEST
         elif (
@@ -1279,7 +1291,8 @@ class Concept(Addressable, DataTyped, ConceptArgs, Mergeable, Namespaced, BaseMo
         elif (
             lineage
             and isinstance(lineage, (Function, BuildFunction))
-            and lineage.operator in (FunctionType.UNNEST, FunctionType.UNION)
+            and lineage.operator
+            in (FunctionType.UNNEST, FunctionType.UNION, FunctionType.DATE_SPINE)
         ):
             return Granularity.MULTI_ROW
         elif lineage and all(
@@ -2125,6 +2138,89 @@ class AlignClause(Namespaced, BaseModel):
         )
 
 
+class DeriveItem(Namespaced, DataTyped, ConceptArgs, Mergeable, BaseModel):
+    expr: Expr
+    name: str
+    namespace: str
+
+    @property
+    def derived_concept(self) -> str:
+        return f"{self.namespace}.{self.name}"
+        # return ConceptRef(
+        #     address=f"{self.namespace}.{self.name}",
+        #     datatype=arg_to_datatype(self.expr),
+        # )
+
+    def with_namespace(self, namespace):
+        return DeriveItem.model_construct(
+            expr=(self.expr.with_namespace(namespace) if self.expr else None),
+            name=self.name,
+            namespace=namespace,
+        )
+
+    def with_merge(
+        self, source: Concept, target: Concept, modifiers: List[Modifier]
+    ) -> "DeriveItem":
+        return DeriveItem.model_construct(
+            expr=(
+                self.expr.with_merge(source, target, modifiers)
+                if isinstance(self.expr, Mergeable)
+                else self.expr
+            ),
+            name=self.name,
+            namespace=self.namespace,
+        )
+
+    def with_reference_replacement(self, source, target):
+        return DeriveItem.model_construct(
+            expr=(
+                self.expr.with_reference_replacement(source, target)
+                if isinstance(self.expr, Mergeable)
+                else self.expr
+            ),
+            name=self.name,
+            namespace=self.namespace,
+        )
+
+
+class DeriveClause(Mergeable, Namespaced, BaseModel):
+    items: List[DeriveItem]
+
+    def with_namespace(self, namespace: str) -> "DeriveClause":
+        return DeriveClause.model_construct(
+            items=[
+                x.with_namespace(namespace) if isinstance(x, Namespaced) else x
+                for x in self.items
+            ]
+        )
+
+    def with_merge(
+        self, source: Concept, target: Concept, modifiers: List[Modifier]
+    ) -> "DeriveClause":
+        return DeriveClause.model_construct(
+            items=[
+                (
+                    x.with_merge(source, target, modifiers)
+                    if isinstance(x, Mergeable)
+                    else x
+                )
+                for x in self.items
+            ]
+        )
+
+    def with_reference_replacement(self, source, target):
+        return DeriveClause.model_construct(
+            items=[
+                (
+                    x.with_reference_replacement(source, target)
+                    if isinstance(x, Mergeable)
+                    else x
+                )
+                for x in self.items
+            ]
+        )
+
+
 class SelectLineage(Mergeable, Namespaced, BaseModel):
     selection: List[ConceptRef]
     hidden_components: set[str]
@@ -2169,15 +2265,40 @@ class SelectLineage(Mergeable, Namespaced, BaseModel):
             ),
         )
 
+    def with_namespace(self, namespace):
+        return SelectLineage.model_construct(
+            selection=[x.with_namespace(namespace) for x in self.selection],
+            hidden_components=self.hidden_components,
+            local_concepts={
+                x: y.with_namespace(namespace) for x, y in self.local_concepts.items()
+            },
+            order_by=self.order_by.with_namespace(namespace) if self.order_by else None,
+            limit=self.limit,
+            meta=self.meta,
+            grain=self.grain.with_namespace(namespace),
+            where_clause=(
+                self.where_clause.with_namespace(namespace)
+                if self.where_clause
+                else None
+            ),
+            having_clause=(
+                self.having_clause.with_namespace(namespace)
+                if self.having_clause
+                else None
+            ),
+        )
+
 
 class MultiSelectLineage(Mergeable, ConceptArgs, Namespaced, BaseModel):
     selects: List[SelectLineage]
     align: AlignClause
+
     namespace: str
     order_by: Optional[OrderBy] = None
     limit: Optional[int] = None
     where_clause: Union["WhereClause", None] = Field(default=None)
     having_clause: Union["HavingClause", None] = Field(default=None)
+    derive: DeriveClause | None = None
     hidden_components: set[str]
 
     @property
@@ -2203,6 +2324,11 @@ class MultiSelectLineage(Mergeable, ConceptArgs, Namespaced, BaseModel):
         new = MultiSelectLineage.model_construct(
             selects=[s.with_merge(source, target, modifiers) for s in self.selects],
             align=self.align,
+            derive=(
+                self.derive.with_merge(source, target, modifiers)
+                if self.derive
+                else None
+            ),
             namespace=self.namespace,
             hidden_components=self.hidden_components,
             order_by=(
@@ -2228,6 +2354,7 @@ class MultiSelectLineage(Mergeable, ConceptArgs, Namespaced, BaseModel):
         return MultiSelectLineage.model_construct(
             selects=[c.with_namespace(namespace) for c in self.selects],
             align=self.align.with_namespace(namespace),
+            derive=self.derive.with_namespace(namespace) if self.derive else None,
             namespace=namespace,
             hidden_components=self.hidden_components,
             order_by=self.order_by.with_namespace(namespace) if self.order_by else None,
@@ -2249,6 +2376,9 @@ class MultiSelectLineage(Mergeable, ConceptArgs, Namespaced, BaseModel):
         output = set()
         for item in self.align.items:
             output.add(item.aligned_concept)
+        if self.derive:
+            for ditem in self.derive.items:
+                output.add(ditem.derived_concept)
         return output
 
     @property
@@ -2404,11 +2534,7 @@ class CustomFunctionFactory:
                     target = f"{DEFAULT_NAMESPACE}.{self.function_arguments[idx].name}"
                 else:
                     target = self.function_arguments[idx].name
-                nout = (
-                    nout.with_reference_replacement(target, x)
-                    if isinstance(nout, Mergeable)
-                    else nout
-                )
+                nout = nout.with_reference_replacement(target, x)
         return nout
 
 
@@ -2467,11 +2593,16 @@ class ArgBinding(Namespaced, DataTyped, BaseModel):
 
 class CustomType(BaseModel):
     name: str
-    type: DataType
+    type: DataType | list[DataType]
+    drop_on: list[FunctionType] = Field(default_factory=list)
+    add_on: list[FunctionType] = Field(default_factory=list)
 
     def with_namespace(self, namespace: str) -> "CustomType":
         return CustomType.model_construct(
-            name=address_with_namespace(self.name, namespace), type=self.type
+            name=address_with_namespace(self.name, namespace),
+            type=self.type,
+            drop_on=self.drop_on,
+            add_on=self.add_on,
         )
 
 
