@@ -22,7 +22,7 @@ from pydantic import (
     ConfigDict,
 )
 
-from trilogy.constants import DEFAULT_NAMESPACE, MagicConstants
+from trilogy.constants import DEFAULT_NAMESPACE, VIRTUAL_CONCEPT_PREFIX, MagicConstants
 from trilogy.core.constants import ALL_ROWS_CONCEPT
 from trilogy.core.enums import (
     BooleanOperator,
@@ -48,6 +48,8 @@ from trilogy.core.models.author import (
     Concept,
     ConceptRef,
     Conditional,
+    DeriveClause,
+    DeriveItem,
     FilterItem,
     FuncArgs,
     Function,
@@ -89,6 +91,7 @@ from trilogy.core.models.datasource import (
     RawColumnExpr,
 )
 from trilogy.core.models.environment import Environment
+from trilogy.utility import string_to_hash
 
 # TODO: refactor to avoid these
 if TYPE_CHECKING:
@@ -98,11 +101,40 @@ if TYPE_CHECKING:
 LOGGER_PREFIX = "[MODELS_BUILD]"
 
 
+def generate_concept_name(parent: Any, debug: bool = False) -> str:
+    """Generate a name for a concept based on its parent type and content."""
+    if isinstance(parent, BuildAggregateWrapper):
+        if debug:
+            print(parent)
+        if parent.is_abstract:
+            return f"{VIRTUAL_CONCEPT_PREFIX}_agg_{parent.function.operator.value}_{string_to_hash(str(parent.with_abstract_by()))}"
+        return f"{VIRTUAL_CONCEPT_PREFIX}_agg_{parent.function.operator.value}_{string_to_hash(str(parent))}"
+    elif isinstance(parent, BuildWindowItem):
+        return f"{VIRTUAL_CONCEPT_PREFIX}_window_{parent.type.value}_{string_to_hash(str(parent))}"
+    elif isinstance(parent, BuildFilterItem):
+        return f"{VIRTUAL_CONCEPT_PREFIX}_filter_{string_to_hash(str(parent))}"
+    elif isinstance(parent, BuildFunction):
+        if parent.operator == FunctionType.GROUP:
+            return f"{VIRTUAL_CONCEPT_PREFIX}_group_to_{string_to_hash(str(parent))}"
+        else:
+            return f"{VIRTUAL_CONCEPT_PREFIX}_func_{parent.operator.value}_{string_to_hash(str(parent))}"
+    elif isinstance(parent, BuildParenthetical):
+        return f"{VIRTUAL_CONCEPT_PREFIX}_paren_{string_to_hash(str(parent))}"
+    elif isinstance(parent, BuildComparison):
+        return f"{VIRTUAL_CONCEPT_PREFIX}_comp_{string_to_hash(str(parent))}"
+    else:  # ListWrapper, MapWrapper, or primitive types
+        return f"{VIRTUAL_CONCEPT_PREFIX}_{string_to_hash(str(parent))}"
+
+
 class BuildConceptArgs(ABC):
     @property
     def concept_arguments(self) -> Sequence["BuildConcept"]:
         raise NotImplementedError
 
+    @property
+    def rendered_concept_arguments(self) -> Sequence["BuildConcept"]:
+        return self.concept_arguments
+
     @property
     def existence_arguments(self) -> Sequence[tuple["BuildConcept", ...]]:
         return []
@@ -134,8 +166,9 @@ def concept_is_relevant(
     if concept.purpose in (Purpose.METRIC,):
         if all([c in others for c in concept.grain.components]):
             return False
+    if concept.derivation in (Derivation.UNNEST,):
+        return True
     if concept.derivation in (Derivation.BASIC,):
-
         return any(concept_is_relevant(c, others) for c in concept.concept_arguments)
     if concept.granularity == Granularity.SINGLE_ROW:
         return False
@@ -212,6 +245,52 @@ class LooseBuildConceptList:
         return self.addresses.isdisjoint(other.addresses)
 
 
+@dataclass
+class CanonicalBuildConceptList:
+    concepts: Sequence[BuildConcept]
+
+    @cached_property
+    def addresses(self) -> set[str]:
+        return {s.canonical_address for s in self.concepts}
+
+    @cached_property
+    def sorted_addresses(self) -> List[str]:
+        return sorted(list(self.addresses))
+
+    def __str__(self) -> str:
+        return f"lcl{str(self.sorted_addresses)}"
+
+    def __iter__(self):
+        return iter(self.concepts)
+
+    def __eq__(self, other):
+        if not isinstance(other, CanonicalBuildConceptList):
+            return False
+        return self.addresses == other.addresses
+
+    def issubset(self, other):
+        if not isinstance(other, CanonicalBuildConceptList):
+            return False
+        return self.addresses.issubset(other.addresses)
+
+    def __contains__(self, other):
+        if isinstance(other, str):
+            return other in self.addresses
+        if not isinstance(other, BuildConcept):
+            return False
+        return other.canonical_address in self.addresses
+
+    def difference(self, other):
+        if not isinstance(other, CanonicalBuildConceptList):
+            return False
+        return self.addresses.difference(other.addresses)
+
+    def isdisjoint(self, other):
+        if not isinstance(other, CanonicalBuildConceptList):
+            return False
+        return self.addresses.isdisjoint(other.addresses)
+
+
 class ConstantInlineable(ABC):
 
     def inline_constant(self, concept: BuildConcept):
@@ -241,12 +320,25 @@ def get_concept_arguments(expr) -> List["BuildConcept"]:
     return output
 
 
+def get_rendered_concept_arguments(expr) -> List["BuildConcept"]:
+    output = []
+    if isinstance(expr, BuildConcept):
+        output += [expr]
+
+    elif isinstance(
+        expr,
+        BuildConceptArgs,
+    ):
+        output += expr.rendered_concept_arguments
+    return output
+
+
 @dataclass
 class BuildParamaterizedConceptReference(DataTyped):
     concept: BuildConcept
 
     def __str__(self):
-        return f":{self.concept.address}"
+        return f":{self.concept.address.replace('.', '_')}"
 
     @property
     def safe_address(self) -> str:
@@ -327,6 +419,8 @@ class BuildGrain:
             return False
         if self.components == other.components:
             return True
+        if self.abstract is True and other.abstract is True:
+            return True
         return False
 
     def issubset(self, other: "BuildGrain"):
@@ -409,13 +503,11 @@ class BuildParenthetical(DataTyped, ConstantInlineable, BuildConceptArgs):
 
     @property
     def concept_arguments(self) -> List[BuildConcept]:
-        base: List[BuildConcept] = []
-        x = self.content
-        if isinstance(x, BuildConcept):
-            base += [x]
-        elif isinstance(x, BuildConceptArgs):
-            base += x.concept_arguments
-        return base
+        return get_concept_arguments(self.content)
+
+    @property
+    def rendered_concept_arguments(self) -> Sequence[BuildConcept]:
+        return get_rendered_concept_arguments(self.content)
 
     @property
     def row_arguments(self) -> Sequence[BuildConcept]:
@@ -435,7 +527,7 @@ class BuildParenthetical(DataTyped, ConstantInlineable, BuildConceptArgs):
 
 
 @dataclass
-class BuildConditional(BuildConceptArgs, ConstantInlineable):
+class BuildConditional(DataTyped, BuildConceptArgs, ConstantInlineable):
     left: Union[
         int,
         str,
@@ -562,6 +654,10 @@ class BuildConditional(BuildConceptArgs, ConstantInlineable):
             chunks.append(self)
         return chunks
 
+    @property
+    def output_datatype(self):
+        return DataType.BOOL
+
 
 @dataclass
 class BuildWhereClause(BuildConceptArgs):
@@ -601,7 +697,7 @@ class BuildHavingClause(BuildWhereClause):
 
 
 @dataclass
-class BuildComparison(BuildConceptArgs, ConstantInlineable):
+class BuildComparison(DataTyped, BuildConceptArgs, ConstantInlineable):
 
     left: Union[
         int,
@@ -730,6 +826,10 @@ class BuildComparison(BuildConceptArgs, ConstantInlineable):
             output += self.right.existence_arguments
         return output
 
+    @property
+    def output_datatype(self):
+        return DataType.BOOL
+
 
 @dataclass
 class BuildSubselectComparison(BuildComparison):
@@ -797,6 +897,7 @@ class BuildSubselectComparison(BuildComparison):
 class BuildConcept(Addressable, BuildConceptArgs, DataTyped):
     model_config = ConfigDict(extra="forbid")
     name: str
+    canonical_name: str
     datatype: DataType | ArrayType | StructType | MapType | NumericType | TraitDataType
     purpose: Purpose
     build_is_aggregate: bool
@@ -816,7 +917,6 @@ class BuildConcept(Addressable, BuildConceptArgs, DataTyped):
             BuildMultiSelectLineage,
         ]
     ] = None
-
     keys: Optional[set[str]] = None
     grain: BuildGrain = field(default=None)  # type: ignore
     modifiers: List[Modifier] = field(default_factory=list)  # type: ignore
@@ -866,9 +966,9 @@ class BuildConcept(Addressable, BuildConceptArgs, DataTyped):
     def address(self) -> str:
         return f"{self.namespace}.{self.name}"
 
-    @property
-    def output(self) -> "BuildConcept":
-        return self
+    @cached_property
+    def canonical_address(self) -> str:
+        return f"{self.namespace}.{self.canonical_name}"
 
     @property
     def safe_address(self) -> str:
@@ -878,6 +978,25 @@ class BuildConcept(Addressable, BuildConceptArgs, DataTyped):
             return f"{self.namespace.replace('.','_')}_{self.name.replace('.','_')}"
         return self.name.replace(".", "_")
 
+    def with_materialized_source(self) -> Self:
+        return self.__class__(
+            name=self.name,
+            canonical_name=self.canonical_name,
+            datatype=self.datatype,
+            purpose=self.purpose,
+            metadata=self.metadata,
+            lineage=None,
+            grain=self.grain,
+            namespace=self.namespace,
+            keys=self.keys,
+            modifiers=self.modifiers,
+            pseudonyms=self.pseudonyms,
+            ## bound
+            derivation=Derivation.ROOT,
+            granularity=self.granularity,
+            build_is_aggregate=self.build_is_aggregate,
+        )
+
     def with_grain(self, grain: Optional["BuildGrain" | BuildConcept] = None) -> Self:
         if isinstance(grain, BuildConcept):
             grain = BuildGrain(
@@ -889,6 +1008,7 @@ class BuildConcept(Addressable, BuildConceptArgs, DataTyped):
             )
         return self.__class__(
             name=self.name,
+            canonical_name=self.canonical_name,
             datatype=self.datatype,
             purpose=self.purpose,
             metadata=self.metadata,
@@ -931,6 +1051,7 @@ class BuildConcept(Addressable, BuildConceptArgs, DataTyped):
             grain = self.grain  # type: ignore
         return self.__class__(
             name=self.name,
+            canonical_name=self.canonical_name,
             datatype=self.datatype,
             purpose=self.purpose,
             metadata=self.metadata,
@@ -1040,10 +1161,6 @@ class BuildWindowItem(DataTyped, BuildConceptArgs):
             output += [item]
         return output
 
-    @property
-    def output(self) -> BuildConcept:
-        return self.content
-
     @property
     def output_datatype(self):
         if self.type in (WindowType.RANK, WindowType.ROW_NUMBER):
@@ -1056,7 +1173,7 @@ class BuildWindowItem(DataTyped, BuildConceptArgs):
 
 
 @dataclass
-class BuildCaseWhen(BuildConceptArgs):
+class BuildCaseWhen(DataTyped, BuildConceptArgs):
     comparison: BuildConditional | BuildSubselectComparison | BuildComparison
     expr: "BuildExpr"
 
@@ -1076,15 +1193,22 @@ class BuildCaseWhen(BuildConceptArgs):
             self.expr
         )
 
+    @property
+    def output_datatype(self):
+        return DataType.BOOL
+
 
 @dataclass
-class BuildCaseElse(BuildConceptArgs):
+class BuildCaseElse(DataTyped, BuildConceptArgs):
     expr: "BuildExpr"
 
     @property
     def concept_arguments(self):
         return get_concept_arguments(self.expr)
 
+    def output_datatype(self):
+        return arg_to_datatype(self.expr)
+
 
 @dataclass
 class BuildFunction(DataTyped, BuildConceptArgs):
@@ -1147,6 +1271,16 @@ class BuildFunction(DataTyped, BuildConceptArgs):
             base += get_concept_arguments(arg)
         return base
 
+    @property
+    def rendered_concept_arguments(self) -> List[BuildConcept]:
+        if self.operator == FunctionType.GROUP:
+            base = self.arguments[0]
+            return get_rendered_concept_arguments(base)
+        base = []
+        for arg in self.arguments:
+            base += get_rendered_concept_arguments(arg)
+        return base
+
     @property
     def output_grain(self):
         # aggregates have an abstract grain
@@ -1168,6 +1302,17 @@ class BuildAggregateWrapper(BuildConceptArgs, DataTyped):
         grain_str = [str(c) for c in self.by] if self.by else "abstract"
         return f"{str(self.function)}<{grain_str}>"
 
+    @property
+    def is_abstract(self):
+        if not self.by:
+            return True
+        if all(x.name == ALL_ROWS_CONCEPT for x in self.by):
+            return True
+        return False
+
+    def with_abstract_by(self) -> "BuildAggregateWrapper":
+        return BuildAggregateWrapper(function=self.function, by=[])
+
     @property
     def datatype(self):
         return self.function.datatype
@@ -1211,11 +1356,14 @@ class BuildFilterItem(BuildConceptArgs):
 
     @property
     def concept_arguments(self):
-        if isinstance(self.content, BuildConcept):
-            return [self.content] + self.where.concept_arguments
-        elif isinstance(self.content, BuildConceptArgs):
-            return self.content.concept_arguments + self.where.concept_arguments
-        return self.where.concept_arguments
+        return self.where.concept_arguments + get_concept_arguments(self.content)
+
+    @property
+    def rendered_concept_arguments(self):
+        return (
+            get_rendered_concept_arguments(self.content)
+            + self.where.rendered_concept_arguments
+        )
 
 
 @dataclass
@@ -1236,10 +1384,6 @@ class BuildRowsetItem(DataTyped, BuildConceptArgs):
     def __str__(self):
         return self.__repr__()
 
-    @property
-    def output(self) -> BuildConcept:
-        return self.content
-
     @property
     def output_datatype(self):
         return self.content.datatype
@@ -1267,6 +1411,22 @@ class BuildAlignClause:
     items: List[BuildAlignItem]
 
 
+@dataclass
+class BuildDeriveClause:
+    items: List[BuildDeriveItem]
+
+
+@dataclass
+class BuildDeriveItem:
+    expr: BuildExpr
+    name: str
+    namespace: str = field(default=DEFAULT_NAMESPACE)
+
+    @property
+    def address(self) -> str:
+        return f"{self.namespace}.{self.name}"
+
+
 @dataclass
 class BuildSelectLineage:
     selection: List[BuildConcept]
@@ -1298,12 +1458,16 @@ class BuildMultiSelectLineage(BuildConceptArgs):
     limit: Optional[int] = None
     where_clause: Union["BuildWhereClause", None] = field(default=None)
     having_clause: Union["BuildHavingClause", None] = field(default=None)
+    derive: BuildDeriveClause | None = None
 
     @property
     def derived_concepts(self) -> set[str]:
         output = set()
         for item in self.align.items:
             output.add(item.aligned_concept)
+        if self.derive:
+            for ditem in self.derive.items:
+                output.add(ditem.address)
         return output
 
     @property
@@ -1311,10 +1475,12 @@ class BuildMultiSelectLineage(BuildConceptArgs):
         return self.build_output_components
 
     @property
-    def derived_concept(self) -> set[str]:
-        output = set()
-        for item in self.align.items:
-            output.add(item.aligned_concept)
+    def calculated_derivations(self) -> set[str]:
+        output: set[str] = set()
+        if not self.derive:
+            return output
+        for item in self.derive.items:
+            output.add(item.address)
         return output
 
     @property
@@ -1334,6 +1500,7 @@ class BuildMultiSelectLineage(BuildConceptArgs):
                 for c in x.concepts:
                     if c.address in cte.output_lcl:
                         return c
+
         raise SyntaxError(
             f"Could not find upstream map for multiselect {str(concept)} on cte ({cte})"
         )
@@ -1453,7 +1620,11 @@ class BuildDatasource:
         # if concept.lineage:
         # #     return None
         for x in self.columns:
-            if x.concept == concept or x.concept.with_grain(concept.grain) == concept:
+            if (
+                x.concept.canonical_address == concept.canonical_address
+                or x.concept == concept
+                or x.concept.with_grain(concept.grain) == concept
+            ):
                 if use_raw_name:
                     return x.alias
                 return concept.safe_address
@@ -1668,7 +1839,6 @@ class Factory:
                     valid_inputs=base.valid_inputs,
                     arg_count=base.arg_count,
                 )
-
         new = BuildFunction(
             operator=base.operator,
             arguments=[self.handle_constant(self.build(c)) for c in raw_args],
@@ -1724,9 +1894,20 @@ class Factory:
         return self._build_concept(base)
 
     def _build_concept(self, base: Concept) -> BuildConcept:
+        try:
+            return self.__build_concept(base)
+        except RecursionError as e:
+            raise RecursionError(
+                f"Recursion error building concept {base.address}. This is likely due to a circular reference."
+            ) from e
+
+    def __build_concept(self, base: Concept) -> BuildConcept:
         # TODO: if we are using parameters, wrap it in a new model and use that in rendering
-        if base.address in self.local_concepts:
-            return self.local_concepts[base.address]
+        # this doesn't work for persisted addresses.
+        # we need to early exit if we have it in local concepts, because in that case,
+        # it is built with appropriate grain only in that dictionary
+        # if base.address in self.local_concepts:
+        #     return self.local_concepts[base.address]
         new_lineage, final_grain, _ = base.get_select_grain_and_keys(
             self.grain, self.environment
         )
@@ -1739,8 +1920,20 @@ class Factory:
         granularity = Concept.calculate_granularity(
             derivation, final_grain, build_lineage
         )
-        is_aggregate = Concept.calculate_is_aggregate(build_lineage)
 
+        def calculate_is_aggregate(lineage):
+            if lineage and isinstance(lineage, BuildFunction):
+                if lineage.operator in FunctionClass.AGGREGATE_FUNCTIONS.value:
+                    return True
+            if (
+                lineage
+                and isinstance(lineage, BuildAggregateWrapper)
+                and lineage.function.operator in FunctionClass.AGGREGATE_FUNCTIONS.value
+            ):
+                return True
+            return False
+
+        is_aggregate = calculate_is_aggregate(build_lineage)
         # if this is a pseudonym, we need to look up the base address
         if base.address in self.environment.alias_origin_lookup:
             lookup_address = self.environment.concepts[base.address].address
@@ -1752,8 +1945,12 @@ class Factory:
                 for x in self.pseudonym_map.get(base.address, set())
                 if x != base.address
             }
+
         rval = BuildConcept(
             name=base.name,
+            canonical_name=(
+                generate_concept_name(build_lineage) if build_lineage else base.name
+            ),
             datatype=base.datatype,
             purpose=base.purpose,
             metadata=base.metadata,
@@ -1768,6 +1965,13 @@ class Factory:
             granularity=granularity,
             build_is_aggregate=is_aggregate,
         )
+        if base.address in self.local_concepts:
+            # comp = self.local_concepts[base.address]
+            # if comp.canonical_address != rval.canonical_address:
+            #     raise ValueError(
+            #         f"Conflicting concepts for address {base.address}: {comp.canonical_address} vs {rval.canonical_address} {comp.lineage} vs {rval.lineage}"
+            #     )
+            return self.local_concepts[base.address]
         self.local_concepts[base.address] = rval
         return rval
 
@@ -1785,7 +1989,9 @@ class Factory:
             by = [self.build(x) for x in base.by]
 
         parent: BuildFunction = self._build_function(base.function)  # type: ignore
-        return BuildAggregateWrapper(function=parent, by=by)
+        return BuildAggregateWrapper(
+            function=parent, by=sorted(by, key=lambda x: x.address)
+        )
 
     @build.register
     def _(self, base: ColumnAssignment) -> BuildColumnAssignment:
@@ -1953,6 +2159,28 @@ class Factory:
     def _build_align_clause(self, base: AlignClause) -> BuildAlignClause:
         return BuildAlignClause(items=[self._build_align_item(x) for x in base.items])
 
+    @build.register
+    def _(self, base: DeriveItem) -> BuildDeriveItem:
+        return self._build_derive_item(base)
+
+    def _build_derive_item(self, base: DeriveItem) -> BuildDeriveItem:
+        expr: Concept | FuncArgs = base.expr
+        validation = requires_concept_nesting(expr)
+        if validation:
+            expr, _ = self.instantiate_concept(validation)
+        return BuildDeriveItem(
+            expr=self.build(expr),
+            name=base.name,
+            namespace=base.namespace,
+        )
+
+    @build.register
+    def _(self, base: DeriveClause) -> BuildDeriveClause:
+        return self._build_derive_clause(base)
+
+    def _build_derive_clause(self, base: DeriveClause) -> BuildDeriveClause:
+        return BuildDeriveClause(items=[self.build(x) for x in base.items])
+
     @build.register
     def _(self, base: RowsetItem) -> BuildRowsetItem:
         return self._build_rowset_item(base)
@@ -2002,6 +2230,13 @@ class Factory:
     def _build_tuple_wrapper(self, base: TupleWrapper) -> TupleWrapper:
         return TupleWrapper(val=[self.build(x) for x in base.val], type=base.type)
 
+    @build.register
+    def _(self, base: ListWrapper) -> ListWrapper:
+        return self._build_list_wrapper(base)
+
+    def _build_list_wrapper(self, base: ListWrapper) -> ListWrapper:
+        return ListWrapper([self.build(x) for x in base], type=base.type)
+
     @build.register
     def _(self, base: FilterItem) -> BuildFilterItem:
         return self._build_filter_item(base)
@@ -2046,6 +2281,7 @@ class Factory:
             pseudonym_map=self.pseudonym_map,
         )
         for k, v in base.local_concepts.items():
+
             materialized[k] = factory.build(v)
         where_factory = Factory(
             grain=Grain(),
@@ -2116,6 +2352,7 @@ class Factory:
             base_concept = self.environment.concepts[k]
             x = BuildConcept(
                 name=base_concept.name,
+                canonical_name=base_concept.name,
                 datatype=base_concept.datatype,
                 purpose=base_concept.purpose,
                 build_is_aggregate=False,
@@ -2147,6 +2384,7 @@ class Factory:
             selects=base.selects,
             grain=final_grain,
             align=factory.build(base.align),
+            derive=factory.build(base.derive) if base.derive else None,
             # self.align.with_select_context(
             #     local_build_cache, self.grain, environment
             # ),
@@ -2181,18 +2419,23 @@ class Factory:
         )
 
         for k, v in base.concepts.items():
-            new.concepts[k] = self._build_concept(v)
+            v_build = self._build_concept(v)
+            new.concepts[k] = v_build
+            new.canonical_concepts[v_build.canonical_address] = v_build
         for (
             k,
             d,
         ) in base.datasources.items():
             new.datasources[k] = self._build_datasource(d)
         for k, a in base.alias_origin_lookup.items():
-            new.alias_origin_lookup[k] = self._build_concept(a)
+            a_build = self._build_concept(a)
+            new.alias_origin_lookup[k] = a_build
+            new.canonical_concepts[a_build.canonical_address] = a_build
         # add in anything that was built as a side-effect
         for bk, bv in self.local_concepts.items():
             if bk not in new.concepts:
                 new.concepts[bk] = bv
+                new.canonical_concepts[bv.canonical_address] = bv
         new.gen_concept_list_caches()
         return new
 
