@@ -108,6 +108,11 @@ class BuildEnvironment(BaseModel):
     concepts: Annotated[
         BuildEnvironmentConceptDict, PlainValidator(validate_concepts)
     ] = Field(default_factory=BuildEnvironmentConceptDict)
+
+    canonical_concepts: Annotated[
+        BuildEnvironmentConceptDict, PlainValidator(validate_concepts)
+    ] = Field(default_factory=BuildEnvironmentConceptDict)
+
     datasources: Annotated[
         BuildEnvironmentDatasourceDict, PlainValidator(validate_datasources)
     ] = Field(default_factory=BuildEnvironmentDatasourceDict)
@@ -116,13 +121,24 @@ class BuildEnvironment(BaseModel):
     namespace: str = DEFAULT_NAMESPACE
     cte_name_map: Dict[str, str] = Field(default_factory=dict)
     materialized_concepts: set[str] = Field(default_factory=set)
+    materialized_canonical_concepts: set[str] = Field(default_factory=set)
+    non_partial_materialized_canonical_concepts: set[str] = Field(default_factory=set)
     alias_origin_lookup: Dict[str, BuildConcept] = Field(default_factory=dict)
 
     def gen_concept_list_caches(self) -> None:
-        concrete_addresses = set()
+        concrete_concepts: list[BuildConcept] = []
+        non_partial_concrete_concepts: list[BuildConcept] = []
         for datasource in self.datasources.values():
-            for concept in datasource.output_concepts:
-                concrete_addresses.add(concept.address)
+            for column in datasource.columns:
+                if column.is_complete:
+                    non_partial_concrete_concepts.append(column.concept)
+                concrete_concepts.append(column.concept)
+        concrete_addresses = set([x.address for x in concrete_concepts])
+        canonical_addresses = set([x.canonical_address for x in concrete_concepts])
+        non_partial_canonical_addresses = set(
+            [x.canonical_address for x in non_partial_concrete_concepts]
+        )
+
         self.materialized_concepts = set(
             [
                 c.address
@@ -135,6 +151,30 @@ class BuildEnvironment(BaseModel):
                 if c.address in concrete_addresses
             ],
         )
+        self.materialized_canonical_concepts = set(
+            [
+                c.canonical_address
+                for c in self.concepts.values()
+                if c.canonical_address in canonical_addresses
+            ]
+            + [
+                c.canonical_address
+                for c in self.alias_origin_lookup.values()
+                if c.canonical_address in canonical_addresses
+            ],
+        )
+        self.non_partial_materialized_canonical_concepts = set(
+            [
+                c.canonical_address
+                for c in self.concepts.values()
+                if c.canonical_address in non_partial_canonical_addresses
+            ]
+            + [
+                c.canonical_address
+                for c in self.alias_origin_lookup.values()
+                if c.canonical_address in non_partial_canonical_addresses
+            ],
+        )
 
 
 BuildEnvironment.model_rebuild()
