@@ -369,7 +369,6 @@ class CTE(BaseModel):
     @property
     def group_concepts(self) -> List[BuildConcept]:
         def check_is_not_in_group(c: BuildConcept):
-
             if len(self.source_map.get(c.address, [])) > 0:
                 return False
             if c.derivation == Derivation.ROWSET:
@@ -385,7 +384,12 @@ class CTE(BaseModel):
                 return True
 
             if c.derivation == Derivation.BASIC and c.lineage:
-                if all([check_is_not_in_group(x) for x in c.lineage.concept_arguments]):
+                if all(
+                    [
+                        check_is_not_in_group(x)
+                        for x in c.lineage.rendered_concept_arguments
+                    ]
+                ):
                     return True
                 if (
                     isinstance(c.lineage, BuildFunction)
@@ -500,6 +504,7 @@ class BaseJoin(BaseModel):
     concepts: Optional[List[BuildConcept]] = None
     left_datasource: Optional[Union[BuildDatasource, "QueryDatasource"]] = None
     concept_pairs: list[ConceptPair] | None = None
+    modifiers: List[Modifier] = Field(default_factory=list)
 
     @model_validator(mode="after")
     def validate_join(self) -> "BaseJoin":
@@ -711,8 +716,6 @@ class QueryDatasource(BaseModel):
             f" {[c.address for c in self.output_concepts]} concepts and"
             f" {other.name} with {[c.address for c in other.output_concepts]} concepts"
         )
-        logger.info(self.source_map)
-        logger.info(other.source_map)
 
         merged_datasources: dict[str, Union[BuildDatasource, "QueryDatasource"]] = {}
 
@@ -816,10 +819,7 @@ class QueryDatasource(BaseModel):
                     use_raw_name,
                     force_alias=force_alias,
                 )
-            except ValueError as e:
-                from trilogy.constants import logger
-
-                logger.debug(e)
+            except ValueError:
                 continue
         existing = [c.with_grain(self.grain) for c in self.output_concepts]
         if concept in existing:
@@ -853,6 +853,7 @@ class RecursiveCTE(CTE):
 
         join_gate = BuildConcept(
             name=RECURSIVE_GATING_CONCEPT,
+            canonical_name=RECURSIVE_GATING_CONCEPT,
             namespace=DEFAULT_NAMESPACE,
             grain=recursive_derived.grain,
             build_is_aggregate=False,
@@ -878,6 +879,7 @@ class RecursiveCTE(CTE):
         )
         bottom_join_gate = BuildConcept(
             name=f"{RECURSIVE_GATING_CONCEPT}_two",
+            canonical_name=f"{RECURSIVE_GATING_CONCEPT}_two",
             namespace=DEFAULT_NAMESPACE,
             grain=recursive_derived.grain,
             build_is_aggregate=False,
@@ -903,6 +905,7 @@ class RecursiveCTE(CTE):
         )
         bottom_derivation = BuildConcept(
             name=recursive_derived.name + "_bottom",
+            canonical_name=recursive_derived.canonical_name + "_bottom",
             namespace=recursive_derived.namespace,
             grain=recursive_derived.grain,
             build_is_aggregate=False,
@@ -1108,6 +1111,7 @@ class Join(BaseModel):
     inlined_ctes: set[str] = Field(default_factory=set)
     quote: str | None = None
     condition: BuildConditional | BuildComparison | BuildParenthetical | None = None
+    modifiers: List[Modifier] = Field(default_factory=list)
 
     def inline_cte(self, cte: CTE):
         self.inlined_ctes.add(cte.name)
