@@ -1,4 +1,5 @@
 from .base_optimization import OptimizationRule
+from .hide_unused_concept import HideUnusedConcepts
 from .inline_datasource import InlineDatasource
 from .predicate_pushdown import PredicatePushdown, PredicatePushdownRemove
 
@@ -7,4 +8,5 @@ __all__ = [
     "InlineDatasource",
     "PredicatePushdown",
     "PredicatePushdownRemove",
+    "HideUnusedConcepts",
 ]
