@@ -14,12 +14,13 @@ from trilogy.core.models.build import (
     BuildWhereClause,
 )
 from trilogy.core.models.build_environment import BuildEnvironment
+from trilogy.core.processing.constants import ROOT_DERIVATIONS, SKIPPED_DERIVATIONS
 from trilogy.core.processing.discovery_node_factory import generate_node
 from trilogy.core.processing.discovery_utility import (
     LOGGER_PREFIX,
     depth_to_prefix,
-    get_priority_concept,
-    group_if_required,
+    get_loop_iteration_targets,
+    group_if_required_v2,
 )
 from trilogy.core.processing.discovery_validation import (
     ValidationResult,
@@ -32,54 +33,6 @@ from trilogy.core.processing.nodes import (
 )
 from trilogy.utility import unique
 
-SKIPPED_DERIVATIONS = [
-    Derivation.AGGREGATE,
-    Derivation.FILTER,
-    Derivation.WINDOW,
-    Derivation.UNNEST,
-    Derivation.RECURSIVE,
-    Derivation.ROWSET,
-    Derivation.BASIC,
-    Derivation.GROUP_TO,
-    Derivation.MULTISELECT,
-    Derivation.UNION,
-]
-
-ROOT_DERIVATIONS = [Derivation.ROOT, Derivation.CONSTANT]
-
-
-def generate_candidates_restrictive(
-    priority_concept: BuildConcept,
-    candidates: list[BuildConcept],
-    exhausted: set[str],
-    depth: int,
-    conditions: BuildWhereClause | None = None,
-) -> tuple[list[BuildConcept], BuildWhereClause | None]:
-    local_candidates = [
-        x
-        for x in list(candidates)
-        if x.address not in exhausted
-        and x.granularity != Granularity.SINGLE_ROW
-        and x.address not in priority_concept.pseudonyms
-        and priority_concept.address not in x.pseudonyms
-    ]
-
-    # if it's single row, joins are irrelevant. Fetch without keys.
-    if priority_concept.granularity == Granularity.SINGLE_ROW:
-        return [], conditions
-
-    if conditions and priority_concept.derivation in ROOT_DERIVATIONS:
-        logger.info(
-            f"{depth_to_prefix(depth)}{LOGGER_PREFIX} Injecting additional conditional row arguments as all remaining concepts are roots or constant"
-        )
-        # otherwise, we can ignore the conditions now that we've injected inputs
-        return (
-            unique(list(conditions.row_arguments) + local_candidates, "address"),
-            None,
-        )
-
-    return local_candidates, conditions
-
 
 def append_existence_check(
     node: StrategyNode,
@@ -250,51 +203,13 @@ def initialize_loop_context(
     )
 
 
-def evaluate_loop_conditions(
-    context: LoopContext, priority_concept: BuildConcept
-) -> BuildWhereClause | None:
-    # filter evaluation
-    # always pass the filter up when we aren't looking at all filter inputs
-    # or there are any non-filter complex types
-    if context.conditions:
-        should_evaluate_filter_on_this_level_not_push_down = all(
-            [
-                x.address in context.mandatory_list
-                for x in context.conditions.row_arguments
-            ]
-        ) and not any(
-            [
-                x.derivation not in ROOT_DERIVATIONS + [Derivation.BASIC]
-                for x in context.mandatory_list
-                if x.address not in context.conditions.row_arguments
-            ]
-        )
-    else:
-        should_evaluate_filter_on_this_level_not_push_down = True
-    local_conditions = (
-        context.conditions
-        if context.conditions
-        and not context.must_evaluate_condition_on_this_level_not_push_down
-        and not should_evaluate_filter_on_this_level_not_push_down
-        else None
-    )
-    # but if it's not basic, and it's not condition;
-    # we do need to push it down (and have another layer of filter evaluation)
-    # to ensure filtering happens before something like a SUM
-    if (
-        context.conditions
-        and priority_concept.derivation not in ROOT_DERIVATIONS + [Derivation.BASIC]
-        and priority_concept.address not in context.conditions.row_arguments
-    ):
-        logger.info(
-            f"{depth_to_prefix(context.depth)}{LOGGER_PREFIX} Force including conditions in {priority_concept.address} to push filtering above complex condition that is not condition member or parent"
-        )
-        local_conditions = context.conditions
-    return local_conditions
-
-
 def check_for_early_exit(
-    complete, partial, missing, context: LoopContext, priority_concept: BuildConcept
+    complete: ValidationResult,
+    found: set[str],
+    partial: set[str],
+    missing: set[str],
+    context: LoopContext,
+    priority_concept: BuildConcept,
 ) -> bool:
     if complete == ValidationResult.INCOMPLETE_CONDITION:
         cond_dict = {str(node): node.preexisting_conditions for node in context.stack}
@@ -319,8 +234,18 @@ def check_for_early_exit(
                 f"{depth_to_prefix(context.depth)}{LOGGER_PREFIX} Breaking as we have attempted all nodes"
             )
             return True
+        elif all(
+            [
+                x.address in found and x.address not in partial
+                for x in context.mandatory_list
+            ]
+        ):
+            logger.info(
+                f"{depth_to_prefix(context.depth)}{LOGGER_PREFIX} Breaking as we have found all mandatory nodes without partials"
+            )
+            return True
         logger.info(
-            f"{depth_to_prefix(context.depth)}{LOGGER_PREFIX} Found complete stack with partials {partial}, continuing search, attempted {context.attempted} all {len(context.mandatory_list)}"
+            f"{depth_to_prefix(context.depth)}{LOGGER_PREFIX} Found complete stack with partials {partial}, continuing search, attempted {context.attempted} of total {len(context.mandatory_list)}."
         )
     else:
         logger.info(
@@ -338,9 +263,6 @@ def check_for_early_exit(
 def generate_loop_completion(context: LoopContext, virtual: set[str]) -> StrategyNode:
     condition_required = True
     non_virtual = [c for c in context.completion_mandatory if c.address not in virtual]
-    non_virtual_output = [
-        c for c in context.original_mandatory if c.address not in virtual
-    ]
     non_virtual_different = len(context.completion_mandatory) != len(
         context.original_mandatory
     )
@@ -368,25 +290,13 @@ def generate_loop_completion(context: LoopContext, virtual: set[str]) -> Strateg
         logger.info(
             f"Condition {context.conditions} not required, parents included filtering! {parent_map}"
         )
+
     if len(context.stack) == 1:
         output: StrategyNode = context.stack[0]
         if non_virtual_different:
             logger.info(
-                f"{depth_to_prefix(context.depth)}{LOGGER_PREFIX} Found different non-virtual output concepts ({non_virtual_difference_values}), removing condition injected values by setting outputs to {[x.address for x in output.output_concepts if x.address in non_virtual_output]}"
+                f"{depth_to_prefix(context.depth)}{LOGGER_PREFIX} Found added non-virtual output concepts ({non_virtual_difference_values})"
             )
-            output.set_output_concepts(
-                [
-                    x
-                    for x in output.output_concepts
-                    if x.address in non_virtual_output
-                    or any(c in non_virtual_output for c in x.pseudonyms)
-                ],
-                rebuild=False,
-            )
-
-        logger.info(
-            f"{depth_to_prefix(context.depth)}{LOGGER_PREFIX} Source stack has single node, returning that {type(output)}"
-        )
     else:
         logger.info(
             f"{depth_to_prefix(context.depth)}{LOGGER_PREFIX} wrapping multiple parent nodes {[type(x) for x in context.stack]} in merge node"
@@ -416,14 +326,26 @@ def generate_loop_completion(context: LoopContext, virtual: set[str]) -> Strateg
     logger.info(
         f"{depth_to_prefix(context.depth)}{LOGGER_PREFIX} Graph is connected, returning {type(output)} node output {[x.address for x in output.usable_outputs]} partial {[c.address for c in output.partial_concepts or []]} with {context.conditions}"
     )
+
     if condition_required and context.conditions and non_virtual_different:
         logger.info(
             f"{depth_to_prefix(context.depth)}{LOGGER_PREFIX} Conditions {context.conditions} were injected, checking if we need a group to restore grain"
         )
-        return group_if_required(
-            output, context.original_mandatory, context.environment
+        return group_if_required_v2(
+            output,
+            context.original_mandatory,
+            context.environment,
+            non_virtual_difference_values,
+            depth=context.depth,
         )
-    return output
+
+    return group_if_required_v2(
+        output,
+        context.original_mandatory,
+        context.environment,
+        non_virtual_difference_values,
+        depth=context.depth,
+    )
 
 
 def _search_concepts(
@@ -446,6 +368,7 @@ def _search_concepts(
         conditions=conditions,
     )
 
+    # if we get a can
     if candidate:
         return candidate
     context = initialize_loop_context(
@@ -457,30 +380,24 @@ def _search_concepts(
         accept_partial=accept_partial,
         conditions=conditions,
     )
-
+    partial: set[str] = set()
+    virtual: set[str] = set()
+    complete = ValidationResult.INCOMPLETE
     while context.incomplete:
-
-        priority_concept = get_priority_concept(
-            context.mandatory_list,
-            context.attempted,
-            found_concepts=context.found,
+        priority_concept, candidate_list, local_conditions = get_loop_iteration_targets(
+            mandatory=context.mandatory_list,
+            conditions=context.conditions,
+            attempted=context.attempted,
+            force_conditions=context.must_evaluate_condition_on_this_level_not_push_down,
+            found=context.found,
+            partial=partial,
             depth=depth,
+            materialized_canonical=(
+                environment.non_partial_materialized_canonical_concepts
+                if not accept_partial
+                else environment.materialized_canonical_concepts
+            ),
         )
-
-        local_conditions = evaluate_loop_conditions(context, priority_concept)
-
-        candidates = [
-            c for c in context.mandatory_list if c.address != priority_concept.address
-        ]
-        # the local conditions list may be overriden if we end up injecting conditions
-        candidate_list, local_conditions = generate_candidates_restrictive(
-            priority_concept,
-            candidates,
-            context.skip,
-            depth=depth,
-            conditions=local_conditions,
-        )
-
         logger.info(
             f"{depth_to_prefix(depth)}{LOGGER_PREFIX} priority concept is {str(priority_concept)} derivation {priority_concept.derivation} granularity {priority_concept.granularity} with conditions {local_conditions}"
         )
@@ -518,7 +435,7 @@ def _search_concepts(
         # assign
         context.found = found_c
         early_exit = check_for_early_exit(
-            complete, partial, missing_c, context, priority_concept
+            complete, found_c, partial, missing_c, context, priority_concept
         )
         if early_exit:
             break
@@ -588,4 +505,4 @@ def source_query_concepts(
     logger.info(
         f"{depth_to_prefix(0)}{LOGGER_PREFIX} final concepts are {[x.address for x in final]}"
     )
-    return group_if_required(root, output_concepts, environment)
+    return group_if_required_v2(root, output_concepts, environment, depth=0)
