@@ -4,7 +4,6 @@ from trilogy.constants import logger
 from trilogy.core.models.build import (
     BuildConcept,
     BuildFilterItem,
-    BuildGrain,
     BuildWhereClause,
 )
 from trilogy.core.models.build_environment import BuildEnvironment
@@ -97,6 +96,8 @@ def build_parent_concepts(
                 continue
         elif global_filter_is_local_filter:
             same_filter_optional.append(x)
+            # also append it to the parent row concepts
+            parent_row_concepts.append(x)
 
     # sometimes, it's okay to include other local optional above the filter
     # in case it is, prep our list
@@ -205,11 +206,16 @@ def gen_filter_node(
             f"{padding(depth)}{LOGGER_PREFIX} filter node row parents {[x.address for x in parent_row_concepts]} could not be found"
         )
         return None
+    else:
+        logger.info(
+            f"{padding(depth)}{LOGGER_PREFIX} filter node has row parents {[x.address for x in parent_row_concepts]} from node with output [{[x.address for x in row_parent.output_concepts]}] partial {row_parent.partial_concepts}"
+        )
     if global_filter_is_local_filter:
         logger.info(
             f"{padding(depth)}{LOGGER_PREFIX} filter node conditions match global conditions adding row parent {row_parent.output_concepts} with condition {where.conditional}"
         )
         row_parent.add_parents(core_parent_nodes)
+        # all local optional will be in the parent already, so we can set outputs
         row_parent.set_output_concepts([concept] + local_optional)
         return row_parent
     if optimized_pushdown:
@@ -238,21 +244,14 @@ def gen_filter_node(
         if not parent.preexisting_conditions == where.conditional:
             parent.add_condition(where.conditional)
         parent.add_existence_concepts(flattened_existence, False)
-        parent.grain = BuildGrain.from_concepts(
-            parent.output_concepts,
-            environment=environment,
-        )
+        # parent.grain = BuildGrain.from_concepts(
+        #     parent.output_concepts,
+        #     environment=environment,
+        # )
         parent.rebuild_cache()
         filter_node = parent
     else:
         core_parent_nodes.append(row_parent)
-        filters = [concept] + same_filter_optional
-        parents_for_grain = [
-            x.lineage.content
-            for x in filters
-            if isinstance(x.lineage, BuildFilterItem)
-            and isinstance(x.lineage.content, BuildConcept)
-        ]
         filter_node = FilterNode(
             input_concepts=unique(
                 parent_row_concepts + flattened_existence,
@@ -261,9 +260,6 @@ def gen_filter_node(
             output_concepts=[concept] + same_filter_optional + parent_row_concepts,
             environment=environment,
             parents=core_parent_nodes,
-            grain=BuildGrain.from_concepts(
-                parents_for_grain + parent_row_concepts, environment=environment
-            ),
             preexisting_conditions=conditions.conditional if conditions else None,
         )
 
