@@ -20,9 +20,6 @@ from trilogy.core.models.build import (
     BuildWhereClause,
 )
 from trilogy.core.models.build_environment import BuildEnvironment
-from trilogy.core.processing.discovery_utility import (
-    group_if_required,
-)
 from trilogy.core.processing.nodes import History, MergeNode, StrategyNode
 from trilogy.core.processing.utility import padding
 from trilogy.utility import unique
@@ -53,9 +50,10 @@ def extract_address(node: str):
 
 
 def extract_concept(node: str, env: BuildEnvironment):
-    if node in env.alias_origin_lookup:
-        return env.alias_origin_lookup[node]
-    return env.concepts[node]
+    # removing this as part of canonical mapping
+    # if node in env.alias_origin_lookup:
+    #     return env.alias_origin_lookup[node]
+    return env.canonical_concepts[node]
 
 
 def filter_unique_graphs(graphs: list[list[str]]) -> list[list[str]]:
@@ -270,7 +268,7 @@ def determine_induced_minimal_nodes(
         if edge[1] in final.nodes and edge[0].startswith("ds~"):
             ds_name = extract_address(edge[0])
             ds = environment.datasources[ds_name]
-            concept = environment.concepts[extract_address(edge[1])]
+            concept = environment.canonical_concepts[extract_address(edge[1])]
             if concept.address in [x.address for x in ds.partial_concepts]:
                 if not accept_partial:
                     continue
@@ -367,26 +365,29 @@ def filter_relevant_subgraphs(
     ]
 
 
-def filter_duplicate_subgraphs(
-    subgraphs: list[list[BuildConcept]], environment
-) -> list[list[BuildConcept]]:
-    seen: list[set[str]] = []
+# 2025-11-18 - removing this as it was causing us to drop
+# partial concept required parents
+# but leaving here for possible future use
+# def filter_duplicate_subgraphs(
+#     subgraphs: list[list[BuildConcept]], environment
+# ) -> list[list[BuildConcept]]:
+#     seen: list[set[str]] = []
 
-    for graph in subgraphs:
-        seen.append(
-            canonicalize_addresses(set([x.address for x in graph]), environment)
-        )
-    final = []
-    # sometimes w can get two subcomponents that are the same
-    # due to alias resolution
-    # if so, drop any that are strict subsets.
-    for graph in subgraphs:
-        logger.info(f"Checking graph {graph} for duplicates in {seen}")
-        set_x = canonicalize_addresses(set([x.address for x in graph]), environment)
-        if any([set_x.issubset(y) and set_x != y for y in seen]):
-            continue
-        final.append(graph)
-    return final
+#     for graph in subgraphs:
+#         seen.append(
+#             canonicalize_addresses(set([x.address for x in graph]), environment)
+#         )
+#     final = []
+#     # sometimes w can get two subcomponents that are the same
+#     # due to alias resolution
+#     # if so, drop any that are strict subsets.
+#     for graph in subgraphs:
+#         logger.info(f"{LOGGER_PREFIX} Checking graph {graph} for duplicates in {seen}")
+#         set_x = canonicalize_addresses(set([x.address for x in graph]), environment)
+#         if any([set_x.issubset(y) and set_x != y for y in seen]):
+#             continue
+#         final.append(graph)
+#     return final
 
 
 def resolve_weak_components(
@@ -492,9 +493,8 @@ def resolve_weak_components(
         if not sub_component:
             continue
         subgraphs.append(sub_component)
-    final = filter_duplicate_subgraphs(subgraphs, environment)
-    return final
-    # return filter_relevant_subgraphs(subgraphs)
+
+    return subgraphs
 
 
 def subgraphs_to_merge_node(
@@ -510,7 +510,7 @@ def subgraphs_to_merge_node(
     search_conditions: BuildWhereClause | None = None,
     enable_early_exit: bool = True,
 ):
-    target_grain = BuildGrain.from_concepts(output_concepts, environment=environment)
+
     parents: List[StrategyNode] = []
     logger.info(
         f"{padding(depth)}{LOGGER_PREFIX} fetching subgraphs {[[c.address for c in subgraph] for subgraph in concept_subgraphs]}"
@@ -555,7 +555,7 @@ def subgraphs_to_merge_node(
             f"{padding(depth)}{LOGGER_PREFIX} only one parent node, exiting early w/ {[c.address for c in parents[0].output_concepts]}"
         )
         parent = parents[0]
-        return group_if_required(parent, output_concepts, environment)
+        return parent
 
     rval = MergeNode(
         input_concepts=unique(input_c, "address"),
@@ -563,7 +563,6 @@ def subgraphs_to_merge_node(
         environment=environment,
         parents=parents,
         depth=depth,
-        grain=target_grain,
         # hidden_concepts=[]
         # conditions=conditions,
         # conditions=search_conditions.conditional,
