@@ -9,6 +9,7 @@ from trilogy.core.graph_models import (
     ReferenceGraph,
     concept_to_node,
     get_graph_exact_match,
+    prune_sources_for_aggregates,
     prune_sources_for_conditions,
 )
 from trilogy.core.models.build import (
@@ -16,7 +17,7 @@ from trilogy.core.models.build import (
     BuildDatasource,
     BuildGrain,
     BuildWhereClause,
-    LooseBuildConceptList,
+    CanonicalBuildConceptList,
 )
 from trilogy.core.models.build_environment import BuildEnvironment
 from trilogy.core.processing.node_generators.select_helpers.datasource_injection import (
@@ -81,9 +82,10 @@ def subgraph_is_complete(
     # Check if all targets are present in mapped nodes
     mapped = {mapping.get(n, n) for n in nodes}
     if not targets.issubset(mapped):
-        # logger.info(
-        #     f"Subgraph {nodes} is not complete, missing targets {targets} - mapped {mapped}"
-        # )
+        missing = targets - mapped
+        logger.debug(
+            f"Subgraph {nodes} is not complete, missing targets {missing} - mapped {mapped}"
+        )
         return False
 
     # Check if at least one concept node has a datasource edge
@@ -129,14 +131,15 @@ def create_pruned_concept_graph(
             g.add_edge(node_address, cnode)
             g.add_edge(cnode, node_address)
     prune_sources_for_conditions(g, accept_partial, conditions)
-    target_addresses = set([c.address for c in all_concepts])
+    prune_sources_for_aggregates(g, all_concepts, logger)
+    target_addresses = set([c.canonical_address for c in all_concepts])
     concepts: dict[str, BuildConcept] = orig_g.concepts
     datasource_map: dict[str, BuildDatasource] = orig_g.datasources
     relevant_concepts_pre = {
-        n: x.address
+        n: x.canonical_address
         for n in g.nodes()
         # filter out synonyms
-        if (x := concepts.get(n, None)) and x.address in target_addresses
+        if (x := concepts.get(n, None)) and x.canonical_address in target_addresses
     }
 
     relevant_concepts: list[str] = list(relevant_concepts_pre.keys())
@@ -165,7 +168,7 @@ def create_pruned_concept_graph(
         if any([[n, x] in g.edges for x in relevant_concepts]):
             relevent_datasets.append(n)
             continue
-
+    logger.debug(f"Relevant datasets after pruning: {relevent_datasets}")
     # for injecting extra join concepts that are shared between datasets
     # use the original graph, pre-partial pruning
     for n in orig_g.concepts:
@@ -224,6 +227,72 @@ def create_pruned_concept_graph(
     return g
 
 
+# def deduplicate_nodes(subgraph: nx.DiGraph, nodes: list[str], partial_map: dict[str, list[str]], depth: int) -> list[str]:
+#     """
+#     Remove duplicate datasource nodes that are connected to the same concepts
+#     and have the same partial state, keeping the one with the most unique concepts.
+
+#     Args:
+#         subgraph: NetworkX DiGraph containing the nodes and edges
+#         nodes: List of node names to deduplicate
+#         partial_map: Map of datasource to partial nodes
+
+#     Returns:
+#         List of deduplicated node names
+#     """
+#     # Filter for datasource nodes only
+#     ds_nodes = [node for node in nodes if node.startswith("ds~")]
+#     non_ds_nodes = [node for node in nodes if not node.startswith("ds~")]
+
+#     if len(ds_nodes) <= 1:
+#         return nodes  # No deduplication needed
+
+#     # Build a map of each datasource to its connected concepts and partial state
+#     ds_info = {}
+
+#     for ds_node in ds_nodes:
+#         # Get connected concept nodes (nodes starting with "c~")
+#         connected_concepts = set()
+#         for neighbor in subgraph.neighbors(ds_node):
+#             if neighbor.startswith("c~"):
+#                 connected_concepts.add(neighbor)
+
+#         # Get partial state for this datasource
+#         partial_state = tuple(sorted(partial_map.get(ds_node, [])))
+
+#         ds_info[ds_node] = {
+#             'concepts': connected_concepts,
+#             'partial_state': partial_state
+#         }
+
+#     # Find datasources to remove (those that are subsets of others)
+#     nodes_to_remove = set()
+#     logger.info('LOOK HERE')
+#     logger.info(ds_info)
+#     for ds_a, info_a in ds_info.items():
+#         for ds_b, info_b in ds_info.items():
+#             if ds_a != ds_b and ds_a not in nodes_to_remove:
+#                 # Check if ds_a is a subset of ds_b (same partial state and concepts are subset)
+#                 if (info_a['partial_state'] == info_b['partial_state'] and
+#                     info_a['concepts'].issubset(info_b['concepts']) and
+#                     len(info_a['concepts']) < len(info_b['concepts'])):
+#                     # ds_a connects to fewer concepts than ds_b, so remove ds_a
+#                     nodes_to_remove.add(ds_a)
+#                 elif (info_a['partial_state'] == info_b['partial_state'] and
+#                       info_a['concepts'] == info_b['concepts']):
+#                     # Exact same concepts and partial state - keep one arbitrarily
+#                     # (keep the lexicographically smaller one for consistency)
+#                     if ds_a > ds_b:
+#                         nodes_to_remove.add(ds_a)
+
+#     # Keep datasource nodes that weren't marked for removal
+#     logger.info(f"{padding(depth)}{LOGGER_PREFIX} Removing duplicate datasource nodes: {nodes_to_remove}")
+#     deduplicated_ds_nodes = [ds for ds in ds_nodes if ds not in nodes_to_remove]
+
+#     # Return deduplicated datasource nodes plus all non-datasource nodes
+#     return deduplicated_ds_nodes + non_ds_nodes
+
+
 def resolve_subgraphs(
     g: ReferenceGraph,
     relevant: list[BuildConcept],
@@ -242,6 +311,8 @@ def resolve_subgraphs(
     that can evolve in the future but is currently based on datasource
     cardinality."""
     datasources = [n for n in g.nodes if n.startswith("ds~")]
+    canonical_relevant = set([c.canonical_address for c in relevant])
+    canonical_map = {c.canonical_address: c.address for c in relevant}
     subgraphs: dict[str, list[str]] = {
         ds: list(set(list(nx.all_neighbors(g, ds)))) for ds in datasources
     }
@@ -250,11 +321,15 @@ def resolve_subgraphs(
     grain_length = get_graph_grains(g)
     concepts: dict[str, BuildConcept] = g.concepts
     non_partial_map = {
-        ds: [concepts[c].address for c in subgraphs[ds] if c not in partial_map[ds]]
+        ds: [
+            concepts[c].canonical_address
+            for c in subgraphs[ds]
+            if c not in partial_map[ds]
+        ]
         for ds in datasources
     }
     concept_map = {
-        ds: [concepts[c].address for c in subgraphs[ds]] for ds in datasources
+        ds: [concepts[c].canonical_address for c in subgraphs[ds]] for ds in datasources
     }
     pruned_subgraphs = {}
 
@@ -312,11 +387,14 @@ def resolve_subgraphs(
 
     final_nodes: set[str] = set([n for v in pruned_subgraphs.values() for n in v])
     relevant_concepts_pre = {
-        n: x.address
+        n: x.canonical_address
         for n in g.nodes()
         # filter out synonyms
-        if (x := concepts.get(n, None)) and x.address in relevant
+        if (x := concepts.get(n, None)) and x.canonical_address in canonical_relevant
     }
+    logger.debug(
+        f"{padding(depth)}{LOGGER_PREFIX} Final nodes before relevance pruning: {final_nodes}"
+    )
     for node in final_nodes:
         keep = True
         if node.startswith("c~") and node not in relevant_concepts_pre:
@@ -334,7 +412,8 @@ def resolve_subgraphs(
                 f"{padding(depth)}{LOGGER_PREFIX} Pruning node {node} as irrelevant after subgraph resolution"
             )
             pruned_subgraphs = {
-                k: [n for n in v if n != node] for k, v in pruned_subgraphs.items()
+                canonical_map.get(k, k): [n for n in v if n != node]
+                for k, v in pruned_subgraphs.items()
             }
 
     return pruned_subgraphs
@@ -350,17 +429,22 @@ def create_datasource_node(
 ) -> tuple[StrategyNode, bool]:
 
     target_grain = BuildGrain.from_concepts(all_concepts, environment=environment)
+    # datasource grain may have changed since reference graph creation
+    datasource_grain = BuildGrain.from_concepts(
+        datasource.grain.components, environment=environment
+    )
+    # datasource_grain = datasource.grain
     force_group = False
-    if not datasource.grain.issubset(target_grain):
+    if not datasource_grain.issubset(target_grain):
         logger.info(
-            f"{padding(depth)}{LOGGER_PREFIX}_DS_NODE Select node must be wrapped in group, {datasource.grain} not subset of target grain {target_grain}"
+            f"{padding(depth)}{LOGGER_PREFIX}_DS_NODE Select node must be wrapped in group, {datasource_grain} not subset of target grain {target_grain} from {all_concepts}"
         )
         force_group = True
     else:
         logger.info(
-            f"{padding(depth)}{LOGGER_PREFIX}_DS_NODE Select node grain {datasource.grain} is subset of target grain {target_grain}, no group required"
+            f"{padding(depth)}{LOGGER_PREFIX}_DS_NODE Select node grain {datasource_grain} is subset of target grain {target_grain}, no group required"
         )
-    if not datasource.grain.components:
+    if not datasource_grain.components:
         force_group = True
     partial_concepts = [
         c.concept
@@ -368,19 +452,28 @@ def create_datasource_node(
         if not c.is_complete and c.concept.address in all_concepts
     ]
 
-    partial_lcl = LooseBuildConceptList(concepts=partial_concepts)
+    partial_lcl = CanonicalBuildConceptList(concepts=partial_concepts)
     nullable_concepts = [
         c.concept
         for c in datasource.columns
         if c.is_nullable and c.concept.address in all_concepts
     ]
 
-    nullable_lcl = LooseBuildConceptList(concepts=nullable_concepts)
+    nullable_lcl = CanonicalBuildConceptList(concepts=nullable_concepts)
     partial_is_full = conditions and (conditions == datasource.non_partial_for)
 
     datasource_conditions = datasource.where.conditional if datasource.where else None
+    all_inputs = [c.concept for c in datasource.columns]
+    canonical_all = CanonicalBuildConceptList(concepts=all_inputs)
+
+    # if we're binding a via a canonical address association
+    # we need to add it here.
+    for x in all_concepts:
+        if x not in all_inputs and x in canonical_all:
+            all_inputs.append(x)
+
     rval = SelectNode(
-        input_concepts=[c.concept for c in datasource.columns],
+        input_concepts=all_inputs,
         output_concepts=sorted(all_concepts, key=lambda x: x.address),
         environment=environment,
         parents=[],
@@ -452,9 +545,10 @@ def create_select_node(
     depth: int,
     conditions: BuildWhereClause | None = None,
 ) -> StrategyNode:
-
     all_concepts = [
-        environment.concepts[extract_address(c)] for c in subgraph if c.startswith("c~")
+        environment.canonical_concepts[extract_address(c)]
+        for c in subgraph
+        if c.startswith("c~")
     ]
 
     if all([c.derivation == Derivation.CONSTANT for c in all_concepts]):
@@ -474,6 +568,7 @@ def create_select_node(
         )
 
     datasource: BuildDatasource = g.datasources[ds_name]
+
     if isinstance(datasource, BuildDatasource):
         bcandidate, force_group = create_datasource_node(
             datasource,
@@ -542,7 +637,7 @@ def gen_select_merge_node(
                 [x.derivation == Derivation.CONSTANT for x in conditions.row_arguments]
             ):
                 logger.info(
-                    f"{padding(depth)}{LOGGER_PREFIX} conditions being passed in to constant node {conditions}, but not all concepts are constants."
+                    f"{padding(depth)}{LOGGER_PREFIX} conditions being passed in to constant node {conditions}, but not all concepts are constants, cannot generate select node."
                 )
                 return None
             else:
@@ -578,7 +673,7 @@ def gen_select_merge_node(
         )
         if pruned_concept_graph:
             logger.info(
-                f"{padding(depth)}{LOGGER_PREFIX} found covering graph w/ partial flag {attempt}"
+                f"{padding(depth)}{LOGGER_PREFIX} found covering graph w/ partial flag {attempt} {list(pruned_concept_graph.nodes)}"
             )
             break
 
