@@ -1,11 +1,12 @@
 from collections import defaultdict
-from dataclasses import dataclass
+from dataclasses import dataclass, field
 from typing import List, Optional
 
 from trilogy.core.enums import (
     BooleanOperator,
     Derivation,
     JoinType,
+    Modifier,
     SourceType,
 )
 from trilogy.core.models.build import (
@@ -257,10 +258,14 @@ class StrategyNode:
         self.partial_lcl = LooseBuildConceptList(concepts=partials)
         return partials
 
-    def add_output_concepts(self, concepts: List[BuildConcept], rebuild: bool = True):
+    def add_output_concepts(
+        self, concepts: List[BuildConcept], rebuild: bool = True, unhide: bool = True
+    ):
         for concept in concepts:
             if concept.address not in self.output_lcl.addresses:
                 self.output_concepts.append(concept)
+            if unhide and concept.address in self.hidden_concepts:
+                self.hidden_concepts.remove(concept.address)
         self.output_lcl = LooseBuildConceptList(concepts=self.output_concepts)
         if rebuild:
             self.rebuild_cache()
@@ -285,12 +290,23 @@ class StrategyNode:
             self.rebuild_cache()
         return self
 
-    def set_output_concepts(self, concepts: List[BuildConcept], rebuild: bool = True):
+    def set_visible_concepts(self, concepts: List[BuildConcept]):
+        for x in self.output_concepts:
+            if x.address not in [c.address for c in concepts]:
+                self.hidden_concepts.add(x.address)
+        return self
+
+    def set_output_concepts(
+        self,
+        concepts: List[BuildConcept],
+        rebuild: bool = True,
+        change_visibility: bool = True,
+    ):
         # exit if no changes
         if self.output_concepts == concepts:
             return self
         self.output_concepts = concepts
-        if self.hidden_concepts:
+        if self.hidden_concepts and change_visibility:
             self.hidden_concepts = set(
                 x for x in self.hidden_concepts if x not in concepts
             )
@@ -425,6 +441,7 @@ class NodeJoin:
     join_type: JoinType
     filter_to_mutual: bool = False
     concept_pairs: list[ConceptPair] | None = None
+    modifiers: List[Modifier] = field(default_factory=list)
 
     def __post_init__(self):
         if self.left_node == self.right_node:
