@@ -169,6 +169,7 @@ class MergeNode(StrategyNode):
                     join_type=join.join_type,
                     concepts=join.concepts,
                     concept_pairs=join.concept_pairs,
+                    modifiers=join.modifiers,
                 )
             )
         return joins
@@ -341,6 +342,7 @@ class MergeNode(StrategyNode):
                 full_join_concepts += join.input_concepts
 
         if self.force_group is True:
+
             force_group = True
         elif self.whole_grain:
             force_group = False
@@ -367,6 +369,14 @@ class MergeNode(StrategyNode):
         nullable_concepts = find_nullable_concepts(
             source_map=source_map, joins=joins, datasources=final_datasets
         )
+        if force_group:
+
+            grain = BuildGrain.from_concepts(
+                self.output_concepts, environment=self.environment
+            )
+            logger.info(
+                f"{self.logging_prefix}{LOGGER_PREFIX} forcing group by to achieve grain {grain}"
+            )
         qds = QueryDatasource(
             input_concepts=unique(self.input_concepts, "address"),
             output_concepts=unique(self.output_concepts, "address"),
