@@ -27,6 +27,7 @@ from trilogy.core.models.build import (
     BuildDatasource,
     BuildFilterItem,
     BuildFunction,
+    BuildGrain,
     BuildParenthetical,
     BuildSubselectComparison,
     BuildWindowItem,
@@ -82,13 +83,138 @@ class JoinOrderOutput:
         return set(self.keys.keys())
 
 
+@dataclass
+class GroupRequiredResponse:
+    target: BuildGrain
+    upstream: BuildGrain
+    required: bool
+
+
+def find_all_connecting_concepts(g: nx.Graph, ds1: str, ds2: str) -> set[str]:
+    """Find all concepts that connect two datasources"""
+    concepts1 = set(g.neighbors(ds1))
+    concepts2 = set(g.neighbors(ds2))
+    return concepts1 & concepts2
+
+
+def get_connection_keys(
+    all_connections: dict[tuple[str, str], set[str]], left: str, right: str
+) -> set[str]:
+    """Get all concepts that connect two datasources"""
+    lookup = sorted([left, right])
+    key: tuple[str, str] = (lookup[0], lookup[1])
+    return all_connections.get(key, set())
+
+
+def get_join_type(
+    left: str,
+    right: str,
+    partials: dict[str, list[str]],
+    nullables: dict[str, list[str]],
+    all_connecting_keys: set[str],
+) -> JoinType:
+    left_is_partial = any(key in partials.get(left, []) for key in all_connecting_keys)
+    left_is_nullable = any(
+        key in nullables.get(left, []) for key in all_connecting_keys
+    )
+    right_is_partial = any(
+        key in partials.get(right, []) for key in all_connecting_keys
+    )
+    right_is_nullable = any(
+        key in nullables.get(right, []) for key in all_connecting_keys
+    )
+
+    left_complete = not left_is_partial and not left_is_nullable
+    right_complete = not right_is_partial and not right_is_nullable
+
+    if not left_complete and not right_complete:
+        join_type = JoinType.FULL
+    elif not left_complete and right_complete:
+        join_type = JoinType.RIGHT_OUTER
+    elif not right_complete and left_complete:
+        join_type = JoinType.LEFT_OUTER
+    else:
+        join_type = JoinType.INNER
+    return join_type
+
+
+def reduce_join_types(join_types: Set[JoinType]) -> JoinType:
+    final_join_type = JoinType.INNER
+    has_full = any([x == JoinType.FULL for x in join_types])
+
+    if has_full:
+        final_join_type = JoinType.FULL
+        return final_join_type
+    has_left = any([x == JoinType.LEFT_OUTER for x in join_types])
+    has_right = any([x == JoinType.RIGHT_OUTER for x in join_types])
+    if has_left and has_right:
+        final_join_type = JoinType.FULL
+    elif has_left:
+        final_join_type = JoinType.LEFT_OUTER
+    elif has_right:
+        final_join_type = JoinType.RIGHT_OUTER
+
+    return final_join_type
+
+
+def ensure_content_preservation(joins: list[JoinOrderOutput]):
+    # ensure that for a join, if we have prior joins that would
+    # introduce nulls, we are controlling for that
+    for idx, review_join in enumerate(joins):
+        predecessors = joins[:idx]
+        if review_join.type == JoinType.FULL:
+            continue
+        has_prior_left = False
+        has_prior_right = False
+        for pred in predecessors:
+            if (
+                pred.type in (JoinType.LEFT_OUTER, JoinType.FULL)
+                and pred.right in review_join.lefts
+            ):
+                has_prior_left = True
+            if pred.type in (JoinType.RIGHT_OUTER, JoinType.FULL) and any(
+                x in review_join.lefts for x in pred.lefts
+            ):
+                has_prior_right = True
+        if has_prior_left and has_prior_right:
+            target = JoinType.FULL
+        elif has_prior_left:
+            target = (
+                JoinType.LEFT_OUTER
+                if review_join.type != JoinType.RIGHT_OUTER
+                else JoinType.FULL
+            )
+        elif has_prior_right:
+            target = (
+                JoinType.RIGHT_OUTER
+                if review_join.type != JoinType.LEFT_OUTER
+                else JoinType.FULL
+            )
+        else:
+            target = review_join.type
+        if review_join.type != target:
+            review_join.type = target
+            continue
+
+
 def resolve_join_order_v2(
     g: nx.Graph, partials: dict[str, list[str]], nullables: dict[str, list[str]]
 ) -> list[JoinOrderOutput]:
     datasources = [x for x in g.nodes if x.startswith("ds~")]
     concepts = [x for x in g.nodes if x.startswith("c~")]
 
+    # Pre-compute all possible connections between datasources
+    all_connections: dict[tuple[str, str], set[str]] = {}
+    for i, ds1 in enumerate(datasources):
+        for ds2 in datasources[i + 1 :]:
+            connecting_concepts = find_all_connecting_concepts(g, ds1, ds2)
+            if connecting_concepts:
+                key = tuple(sorted([ds1, ds2]))
+                all_connections[key] = connecting_concepts
+
     output: list[JoinOrderOutput] = []
+
+    # create our map of pivots, or common join concepts
     pivot_map = {
         concept: [x for x in g.neighbors(concept) if x in datasources]
         for concept in concepts
@@ -100,8 +226,9 @@ def resolve_join_order_v2(
         )
     )
     solo = [x for x in pivot_map if len(pivot_map[x]) == 1]
-    eligible_left = set()
+    eligible_left: set[str] = set()
 
+    # while we have pivots, keep joining them in
     while pivots:
         next_pivots = [
             x for x in pivots if any(y in eligible_left for y in pivot_map[x])
@@ -112,7 +239,7 @@ def resolve_join_order_v2(
         else:
             root = pivots.pop(0)
 
-        # sort so less partials is last and eligible lefts are
+        # sort so less partials is last and eligible lefts are first
         def score_key(x: str) -> tuple[int, int, str]:
             base = 1
             # if it's left, higher weight
@@ -125,79 +252,56 @@ def resolve_join_order_v2(
                 base -= 1
             return (base, len(x), x)
 
-        # get remainig un-joined datasets
+        # get remaining un-joined datasets
         to_join = sorted(
             [x for x in pivot_map[root] if x not in eligible_left], key=score_key
         )
         while to_join:
             # need to sort this to ensure we join on the best match
-            base = sorted(
-                [x for x in pivot_map[root] if x in eligible_left], key=score_key
-            )
+            # but check ALL left in case there are non-pivt keys to join on
+            base = sorted([x for x in eligible_left], key=score_key)
             if not base:
                 new = to_join.pop()
                 eligible_left.add(new)
                 base = [new]
             right = to_join.pop()
             # we already joined it
-            # this could happen if the same pivot is shared with multiple Dses
+            # this could happen if the same pivot is shared with multiple DSes
             if right in eligible_left:
                 continue
+
             joinkeys: dict[str, set[str]] = {}
             # sorting puts the best candidate last for pop
             # so iterate over the reversed list
             join_types = set()
-            for left_candidate in reversed(base):
-                common = nx.common_neighbors(g, left_candidate, right)
 
-                if not common:
+            for left_candidate in reversed(base):
+                # Get all concepts that connect these two datasources
+                all_connecting_keys = get_connection_keys(
+                    all_connections, left_candidate, right
+                )
+
+                if not all_connecting_keys:
                     continue
+
+                # Check if we already have this exact set of keys
                 exists = False
                 for _, v in joinkeys.items():
-                    if v == common:
+                    if v == all_connecting_keys:
                         exists = True
                 if exists:
                     continue
-                left_is_partial = any(
-                    key in partials.get(left_candidate, []) for key in common
-                )
-                left_is_nullable = any(
-                    key in nullables.get(left_candidate, []) for key in common
-                )
-                right_is_partial = any(key in partials.get(right, []) for key in common)
-                # we don't care if left is nullable for join type (just keys), but if we did
-                # left_is_nullable = any(
-                #     key in nullables.get(left_candidate, []) for key in common
-                # )
-                right_is_nullable = any(
-                    key in nullables.get(right, []) for key in common
-                )
-                if left_is_nullable and right_is_nullable:
-                    join_type = JoinType.FULL
-                elif left_is_partial and right_is_partial:
-                    join_type = JoinType.FULL
-                elif left_is_partial:
-                    join_type = JoinType.FULL
-                elif right_is_nullable:
-                    join_type = JoinType.RIGHT_OUTER
-                elif right_is_partial or left_is_nullable:
-                    join_type = JoinType.LEFT_OUTER
-                # we can't inner join if the left was an outer join
-                else:
-                    join_type = JoinType.INNER
 
+                join_type = get_join_type(
+                    left_candidate, right, partials, nullables, all_connecting_keys
+                )
                 join_types.add(join_type)
-                joinkeys[left_candidate] = common
-            final_join_type = JoinType.INNER
-            if any([x == JoinType.FULL for x in join_types]):
-                final_join_type = JoinType.FULL
-            elif any([x == JoinType.LEFT_OUTER for x in join_types]):
-                final_join_type = JoinType.LEFT_OUTER
-            elif any([x == JoinType.RIGHT_OUTER for x in join_types]):
-                final_join_type = JoinType.RIGHT_OUTER
+                joinkeys[left_candidate] = all_connecting_keys
+
+            final_join_type = reduce_join_types(join_types)
+
             output.append(
                 JoinOrderOutput(
-                    # left=left_candidate,
                     right=right,
                     type=final_join_type,
                     keys=joinkeys,
@@ -208,7 +312,6 @@ def resolve_join_order_v2(
     for concept in solo:
         for ds in pivot_map[concept]:
             # if we already have it, skip it
-
             if ds in eligible_left:
                 continue
             # if we haven't had ANY left datasources yet
@@ -216,30 +319,43 @@ def resolve_join_order_v2(
             if not eligible_left:
                 eligible_left.add(ds)
                 continue
-            # otherwise do a full out join
-            output.append(
-                JoinOrderOutput(
-                    # pick random one to be left
-                    left=list(eligible_left)[0],
-                    right=ds,
-                    type=JoinType.FULL,
-                    keys={},
+            # otherwise do a full outer join
+            # Try to find if there are any connecting keys with existing left tables
+            best_left = None
+            best_keys: set[str] = set()
+            for existing_left in eligible_left:
+                connecting_keys = get_connection_keys(
+                    all_connections, existing_left, ds
+                )
+                if connecting_keys and len(connecting_keys) > len(best_keys):
+                    best_left = existing_left
+                    best_keys = connecting_keys
+
+            if best_left and best_keys:
+                output.append(
+                    JoinOrderOutput(
+                        left=best_left,
+                        right=ds,
+                        type=JoinType.FULL,
+                        keys={best_left: best_keys},
+                    )
+                )
+            else:
+                output.append(
+                    JoinOrderOutput(
+                        # pick random one to be left
+                        left=list(eligible_left)[0],
+                        right=ds,
+                        type=JoinType.FULL,
+                        keys={},
+                    )
                 )
-            )
             eligible_left.add(ds)
+
     # only once we have all joins
     # do we know if some inners need to be left outers
-    for review_join in output:
-        if review_join.type in (JoinType.LEFT_OUTER, JoinType.FULL):
-            continue
-        if any(
-            [
-                join.right in review_join.lefts
-                for join in output
-                if join.type in (JoinType.LEFT_OUTER, JoinType.FULL)
-            ]
-        ):
-            review_join.type = JoinType.LEFT_OUTER
+    ensure_content_preservation(output)
+
     return output
 
 
@@ -344,7 +460,9 @@ def resolve_instantiated_concept(
     )
 
 
-def reduce_concept_pairs(input: list[ConceptPair]) -> list[ConceptPair]:
+def reduce_concept_pairs(
+    input: list[ConceptPair], right_source: QueryDatasource | BuildDatasource
+) -> list[ConceptPair]:
     left_keys = set()
     right_keys = set()
     for pair in input:
@@ -353,7 +471,10 @@ def reduce_concept_pairs(input: list[ConceptPair]) -> list[ConceptPair]:
         if pair.right.purpose == Purpose.KEY:
             right_keys.add(pair.right.address)
     final: list[ConceptPair] = []
+    seen_right_keys = set()
     for pair in input:
+        if pair.right.address in seen_right_keys:
+            continue
         if (
             pair.left.purpose == Purpose.PROPERTY
             and pair.left.keys
@@ -366,7 +487,15 @@ def reduce_concept_pairs(input: list[ConceptPair]) -> list[ConceptPair]:
             and pair.right.keys.issubset(right_keys)
         ):
             continue
+
+        seen_right_keys.add(pair.right.address)
         final.append(pair)
+    all_keys = set([x.right.address for x in final])
+    if right_source.grain.components and right_source.grain.components.issubset(
+        all_keys
+    ):
+        return [x for x in final if x.right.address in right_source.grain.components]
+
     return final
 
 
@@ -435,7 +564,8 @@ def get_node_joins(
                     )
                     for k, v in j.keys.items()
                     for concept in v
-                ]
+                ],
+                ds_node_map[j.right],
             ),
         )
         for j in joins
@@ -659,6 +789,7 @@ def sort_select_output_processed(
                 new_output.append(
                     BuildConcept(
                         name=x.name,
+                        canonical_name=x.name,
                         namespace=x.namespace,
                         pseudonyms={oc.address},
                         datatype=oc.datatype,
