@@ -21,6 +21,7 @@ from trilogy.core.models.author import (
     Concept,
     ConceptRef,
     CustomType,
+    DeriveClause,
     Expr,
     FilterItem,
     Function,
@@ -31,6 +32,7 @@ from trilogy.core.models.author import (
     Metadata,
     MultiSelectLineage,
     OrderBy,
+    Parenthetical,
     SelectLineage,
     UndefinedConcept,
     WhereClause,
@@ -48,7 +50,12 @@ from trilogy.utility import unique
 
 class ConceptTransform(BaseModel):
     function: (
-        Function | FilterItem | WindowItem | AggregateWrapper | FunctionCallWrapper
+        Function
+        | FilterItem
+        | WindowItem
+        | AggregateWrapper
+        | FunctionCallWrapper
+        | Parenthetical
     )
     output: Concept  # this has to be a full concept, as it may not exist in environment
     modifiers: List[Modifier] = Field(default_factory=list)
@@ -102,6 +109,11 @@ class SelectStatement(HasUUID, SelectTypeMixin, BaseModel):
     grain: Grain = Field(default_factory=Grain)
 
     def as_lineage(self, environment: Environment) -> SelectLineage:
+        derived = [
+            x.concept.address
+            for x in self.selection
+            if isinstance(x.content, ConceptTransform)
+        ]
         return SelectLineage(
             selection=[
                 environment.concepts[x.concept.address].reference
@@ -111,7 +123,9 @@ class SelectStatement(HasUUID, SelectTypeMixin, BaseModel):
             limit=self.limit,
             where_clause=self.where_clause,
             having_clause=self.having_clause,
-            local_concepts=self.local_concepts,
+            local_concepts={
+                k: v for k, v in self.local_concepts.items() if k in derived
+            },
             hidden_components=self.hidden_components,
             grain=self.grain,
         )
@@ -190,9 +204,17 @@ class SelectStatement(HasUUID, SelectTypeMixin, BaseModel):
         if self.where_clause:
             for x in self.where_clause.concept_arguments:
                 if isinstance(x, UndefinedConcept):
-                    environment.concepts.raise_undefined(
-                        x.address, x.metadata.line_number if x.metadata else None
-                    )
+                    validate = environment.concepts.get(x.address)
+                    if validate and self.where_clause:
+                        self.where_clause = (
+                            self.where_clause.with_reference_replacement(
+                                x.address, validate.reference
+                            )
+                        )
+                    else:
+                        environment.concepts.raise_undefined(
+                            x.address, x.metadata.line_number if x.metadata else None
+                        )
         all_in_output = [x for x in self.output_components]
         if self.where_clause:
             for cref in self.where_clause.concept_arguments:
@@ -338,11 +360,13 @@ class MultiSelectStatement(HasUUID, SelectTypeMixin, BaseModel):
     local_concepts: Annotated[
         EnvironmentConceptDict, PlainValidator(validate_concepts)
     ] = Field(default_factory=EnvironmentConceptDict)
+    derive: DeriveClause | None = None
 
     def as_lineage(self, environment: Environment):
         return MultiSelectLineage(
             selects=[x.as_lineage(environment) for x in self.selects],
             align=self.align,
+            derive=self.derive,
             namespace=self.namespace,
             # derived_concepts = self.derived_concepts,
             limit=self.limit,
