@@ -4,7 +4,6 @@ from pydantic import BaseModel, Field
 
 from trilogy.core.enums import IOType
 from trilogy.core.models.author import ConceptRef, HavingClause, WhereClause
-from trilogy.core.models.datasource import Address, Datasource
 
 
 class CopyQueryMixin(BaseModel):
@@ -12,15 +11,6 @@ class CopyQueryMixin(BaseModel):
     target_type: IOType
 
 
-class MaterializedDataset(BaseModel):
-    address: Address
-
-
-class PersistQueryMixin(BaseModel):
-    output_to: MaterializedDataset
-    datasource: Datasource
-
-
 class SelectTypeMixin(BaseModel):
     where_clause: Union["WhereClause", None] = Field(default=None)
     having_clause: Union["HavingClause", None] = Field(default=None)
