@@ -33,6 +33,9 @@ class EngineConnection(Protocol):
     def rollback(self):
         raise NotImplementedError()
 
+    def close(self) -> None:
+        return
+
 
 class ExecutionEngine(Protocol):
     pass
@@ -43,6 +46,9 @@ class ExecutionEngine(Protocol):
     def setup(self, env: Environment, connection):
         pass
 
+    def dispose(self, close: bool = True):
+        pass
+
 
 ### Begin default SQLAlchemy implementation
 class SqlAlchemyResult:
