@@ -6,7 +6,7 @@ from sqlalchemy import text
 
 from trilogy.constants import MagicConstants, Rendering, logger
 from trilogy.core.enums import FunctionType, Granularity, IOType, ValidationScope
-from trilogy.core.models.author import Concept, Function
+from trilogy.core.models.author import Comment, Concept, Function
 from trilogy.core.models.build import BuildFunction
 from trilogy.core.models.core import ListWrapper, MapWrapper
 from trilogy.core.models.datasource import Datasource
@@ -47,7 +47,7 @@ from trilogy.dialect.metadata import (
     handle_processed_validate_statement,
     handle_show_statement_outputs,
 )
-from trilogy.engine import ExecutionEngine, ResultProtocol
+from trilogy.engine import EngineConnection, ExecutionEngine, ResultProtocol
 from trilogy.hooks.base_hook import BaseHook
 from trilogy.parser import parse_text
 from trilogy.render import get_dialect_generator
@@ -69,11 +69,24 @@ class Executor(object):
         self.logger = logger
         self.hooks = hooks
         self.generator = get_dialect_generator(self.dialect, rendering)
-        self.connection = self.engine.connect()
+        self.connection = self.connect()
         # TODO: make generic
         if self.dialect == Dialects.DATAFRAME:
             self.engine.setup(self.environment, self.connection)
 
+    def connect(self) -> EngineConnection:
+        self.connection = self.engine.connect()
+        self.connected = True
+        return self.connection
+
+    def close(self):
+        self.engine.dispose(close=True)
+        if self.dialect == Dialects.DUCK_DB:
+            import gc
+
+            gc.collect()
+        self.connected = False
+
     def execute_statement(
         self,
         statement: PROCESSED_STATEMENT_TYPES,
@@ -86,6 +99,10 @@ class Executor(object):
     def execute_query(self, query) -> ResultProtocol | None:
         raise NotImplementedError("Cannot execute type {}".format(type(query)))
 
+    @execute_query.register
+    def _(self, query: Comment) -> ResultProtocol | None:
+        return None
+
     @execute_query.register
     def _(self, query: ConceptDeclarationStatement) -> ResultProtocol | None:
         return handle_concept_declaration(query)
@@ -240,7 +257,9 @@ class Executor(object):
         """generate SQL for execution"""
         _, parsed = parse_text(command, self.environment)
         generatable = [
-            x for x in parsed if isinstance(x, (SelectStatement, PersistStatement))
+            x
+            for x in parsed
+            if isinstance(x, (SelectStatement, PersistStatement, MultiSelectStatement))
         ]
         sql = self.generator.generate_queries(
             self.environment, generatable, hooks=self.hooks
@@ -266,9 +285,22 @@ class Executor(object):
         None,
     ]:
         file = Path(file)
-        with open(file, "r") as f:
-            command = f.read()
-            return self.parse_text_generator(command, persist=persist, root=file)
+        candidates = [file, self.environment.working_path / file]
+        err = None
+        for file in candidates:
+            try:
+                with open(file, "r") as f:
+                    command = f.read()
+                    return self.parse_text_generator(
+                        command, persist=persist, root=file
+                    )
+            except FileNotFoundError as e:
+                if not err:
+                    err = e
+                continue
+        if err:
+            raise err
+        raise FileNotFoundError(f"File {file} not found")
 
     def parse_text(
         self, command: str, persist: bool = False, root: Path | None = None
@@ -365,7 +397,7 @@ class Executor(object):
                 if v.safe_address == param or v.address == param
             ]
         if not matched:
-            raise SyntaxError(f"No concept found for parameter {param}")
+            raise SyntaxError(f"No concept found for parameter {param};")
 
         concept: Concept = matched.pop()
         return self._concept_to_value(concept, local_concepts=local_concepts)
@@ -402,6 +434,9 @@ class Executor(object):
     def execute_text(
         self, command: str, non_interactive: bool = False
     ) -> List[ResultProtocol]:
+        if not self.connected:
+            self.connect()
+
         """Run a trilogy query expressed as text."""
         output: list[ResultProtocol] = []
         # connection = self.engine.connect()
@@ -440,9 +475,20 @@ class Executor(object):
         self, file: str | Path, non_interactive: bool = False
     ) -> List[ResultProtocol]:
         file = Path(file)
-        with open(file, "r") as f:
-            command = f.read()
-        return self.execute_text(command, non_interactive=non_interactive)
+        candidates = [file, self.environment.working_path / file]
+        err = None
+        for file in candidates:
+            if not file.exists():
+                continue
+            with open(file, "r") as f:
+                command = f.read()
+            if file.suffix == ".sql":
+                return [self.execute_raw_sql(command)]
+            else:
+                return self.execute_text(command, non_interactive=non_interactive)
+        if err:
+            raise err
+        raise FileNotFoundError(f"File {file} not found")
 
     def validate_environment(
         self,
