@@ -3,9 +3,7 @@ from typing import Iterable, List, Sequence, Tuple
 
 from lark.tree import Meta
 
-from trilogy.constants import (
-    VIRTUAL_CONCEPT_PREFIX,
-)
+from trilogy.constants import DEFAULT_NAMESPACE, VIRTUAL_CONCEPT_PREFIX
 from trilogy.core.constants import ALL_ROWS_CONCEPT
 from trilogy.core.enums import (
     ConceptSource,
@@ -55,6 +53,23 @@ from trilogy.core.models.environment import Environment
 from trilogy.core.statements.author import RowsetDerivationStatement, SelectStatement
 from trilogy.utility import string_to_hash, unique
 
+ARBITRARY_INPUTS = (
+    AggregateWrapper
+    | FunctionCallWrapper
+    | WindowItem
+    | FilterItem
+    | Function
+    | Parenthetical
+    | ListWrapper
+    | MapWrapper
+    | Comparison
+    | int
+    | float
+    | str
+    | date
+    | bool
+)
+
 
 def process_function_arg(
     arg,
@@ -76,7 +91,7 @@ def process_function_arg(
         # to simplify anonymous function handling
         if (
             arg.operator not in FunctionClass.AGGREGATE_FUNCTIONS.value
-            and arg.operator != FunctionType.UNNEST
+            and arg.operator not in FunctionClass.ONE_TO_MANY.value
         ):
             return arg
         id_hash = string_to_hash(str(arg))
@@ -296,13 +311,18 @@ def concept_is_relevant(
     if concept.purpose in (Purpose.METRIC,):
         if all([c in others for c in concept.grain.components]):
             return False
+    if (
+        concept.derivation in (Derivation.BASIC,)
+        and isinstance(concept.lineage, Function)
+        and concept.lineage.operator == FunctionType.DATE_SPINE
+    ):
+        return True
     if concept.derivation in (Derivation.BASIC,) and isinstance(
         concept.lineage, (Function, CaseWhen)
     ):
         relevant = False
         for arg in concept.lineage.arguments:
             relevant = atom_is_relevant(arg, others, environment) or relevant
-
         return relevant
     if concept.derivation in (Derivation.BASIC,) and isinstance(
         concept.lineage, Parenthetical
@@ -379,6 +399,10 @@ def _get_relevant_parent_concepts(arg) -> tuple[list[ConceptRef], bool]:
         return [x.reference for x in arg.by], True
     elif isinstance(arg, FunctionCallWrapper):
         return get_relevant_parent_concepts(arg.content)
+    elif isinstance(arg, Comparison):
+        left, lflag = get_relevant_parent_concepts(arg.left)
+        right, rflag = get_relevant_parent_concepts(arg.right)
+        return left + right, lflag or rflag
     return get_concept_arguments(arg), False
 
 
@@ -514,7 +538,7 @@ def function_to_concept(
     elif parent.operator == FunctionType.UNION:
         derivation = Derivation.UNION
         granularity = Granularity.MULTI_ROW
-    elif parent.operator == FunctionType.UNNEST:
+    elif parent.operator in FunctionClass.ONE_TO_MANY.value:
         derivation = Derivation.UNNEST
         granularity = Granularity.MULTI_ROW
     elif parent.operator == FunctionType.RECURSE_EDGE:
@@ -625,7 +649,7 @@ def window_item_to_concept(
     fmetadata = metadata or Metadata()
     if not isinstance(parent.content, ConceptRef):
         raise NotImplementedError(
-            f"Window function wiht non ref content {parent.content} not yet supported"
+            f"Window function with non ref content {parent.content} not yet supported"
         )
     bcontent = environment.concepts[parent.content.address]
     if isinstance(bcontent, UndefinedConcept):
@@ -755,6 +779,27 @@ def align_item_to_concept(
     return new
 
 
+def derive_item_to_concept(
+    parent: ARBITRARY_INPUTS,
+    name: str,
+    lineage: MultiSelectLineage,
+    namespace: str | None = None,
+) -> Concept:
+    datatype = arg_to_datatype(parent)
+    grain = Grain()
+    new = Concept(
+        name=name,
+        datatype=datatype,
+        purpose=Purpose.PROPERTY,
+        lineage=lineage,
+        grain=grain,
+        namespace=namespace or DEFAULT_NAMESPACE,
+        granularity=Granularity.MULTI_ROW,
+        derivation=Derivation.MULTISELECT,
+    )
+    return new
+
+
 def rowset_concept(
     orig_address: ConceptRef,
     environment: Environment,
@@ -836,19 +881,7 @@ def rowset_to_concepts(rowset: RowsetDerivationStatement, environment: Environme
 
 
 def generate_concept_name(
-    parent: (
-        AggregateWrapper
-        | FunctionCallWrapper
-        | WindowItem
-        | FilterItem
-        | Function
-        | ListWrapper
-        | MapWrapper
-        | int
-        | float
-        | str
-        | date
-    ),
+    parent: ARBITRARY_INPUTS,
 ) -> str:
     """Generate a name for a concept based on its parent type and content."""
     if isinstance(parent, AggregateWrapper):
@@ -865,24 +898,114 @@ def generate_concept_name(
             return f"{VIRTUAL_CONCEPT_PREFIX}_group_to_{string_to_hash(str(parent))}"
         else:
             return f"{VIRTUAL_CONCEPT_PREFIX}_func_{parent.operator.value}_{string_to_hash(str(parent))}"
+    elif isinstance(parent, Parenthetical):
+        return f"{VIRTUAL_CONCEPT_PREFIX}_paren_{string_to_hash(str(parent))}"
+    elif isinstance(parent, FunctionCallWrapper):
+        return f"{VIRTUAL_CONCEPT_PREFIX}_{parent.name}_{string_to_hash(str(parent))}"
+    elif isinstance(parent, Comparison):
+        return f"{VIRTUAL_CONCEPT_PREFIX}_comp_{string_to_hash(str(parent))}"
     else:  # ListWrapper, MapWrapper, or primitive types
         return f"{VIRTUAL_CONCEPT_PREFIX}_{string_to_hash(str(parent))}"
 
 
+def parenthetical_to_concept(
+    parent: Parenthetical,
+    name: str,
+    namespace: str,
+    environment: Environment,
+    metadata: Metadata | None = None,
+) -> Concept:
+    if isinstance(
+        parent.content,
+        ARBITRARY_INPUTS,
+    ):
+
+        return arbitrary_to_concept(
+            parent.content, environment, namespace, name, metadata
+        )
+    raise NotImplementedError(
+        f"Parenthetical with non-supported content {parent.content} ({type(parent.content)}) not yet supported"
+    )
+
+
+def comparison_to_concept(
+    parent: Comparison,
+    name: str,
+    namespace: str,
+    environment: Environment,
+    metadata: Metadata | None = None,
+):
+    fmetadata = metadata or Metadata()
+
+    pkeys: List[Concept] = []
+    namespace = namespace or environment.namespace
+    is_metric = False
+    ref_args, is_metric = get_relevant_parent_concepts(parent)
+    concrete_args = [environment.concepts[c.address] for c in ref_args]
+    pkeys += [
+        x
+        for x in concrete_args
+        if not x.derivation == Derivation.CONSTANT
+        and not (x.derivation == Derivation.AGGREGATE and not x.grain.components)
+    ]
+    grain: Grain | None = Grain()
+    for x in pkeys:
+        grain += x.grain
+    if parent.operator in FunctionClass.ONE_TO_MANY.value:
+        # if the function will create more rows, we don't know what grain this is at
+        grain = None
+    modifiers = get_upstream_modifiers(pkeys, environment)
+    key_grain: list[str] = []
+    for x in pkeys:
+        # metrics will group to keys, so do not do key traversal
+        if is_metric:
+            key_grain.append(x.address)
+        # otherwse, for row ops, assume keys are transitive
+        elif x.keys:
+            key_grain += [*x.keys]
+        else:
+            key_grain.append(x.address)
+    keys = set(key_grain)
+    if is_metric:
+        purpose = Purpose.METRIC
+    elif not pkeys:
+        purpose = Purpose.CONSTANT
+    else:
+        purpose = Purpose.PROPERTY
+    fmetadata = metadata or Metadata()
+
+    if grain is not None:
+        r = Concept(
+            name=name,
+            datatype=parent.output_datatype,
+            purpose=purpose,
+            lineage=parent,
+            namespace=namespace,
+            keys=keys,
+            modifiers=modifiers,
+            grain=grain,
+            metadata=fmetadata,
+            derivation=Derivation.BASIC,
+            granularity=Granularity.MULTI_ROW,
+        )
+        return r
+
+    return Concept(
+        name=name,
+        datatype=parent.output_datatype,
+        purpose=purpose,
+        lineage=parent,
+        namespace=namespace,
+        keys=keys,
+        modifiers=modifiers,
+        metadata=fmetadata,
+        derivation=Derivation.BASIC,
+        granularity=Granularity.MULTI_ROW,
+    )
+
+
 def arbitrary_to_concept(
-    parent: (
-        AggregateWrapper
-        | FunctionCallWrapper
-        | WindowItem
-        | FilterItem
-        | Function
-        | ListWrapper
-        | MapWrapper
-        | int
-        | float
-        | str
-        | date
-    ),
+    parent: ARBITRARY_INPUTS,
     environment: Environment,
     namespace: str | None = None,
     name: str | None = None,
@@ -938,5 +1061,9 @@ def arbitrary_to_concept(
         )
     elif isinstance(parent, ListWrapper):
         return constant_to_concept(parent, name, namespace, metadata)
+    elif isinstance(parent, Parenthetical):
+        return parenthetical_to_concept(parent, name, namespace, environment, metadata)
+    elif isinstance(parent, Comparison):
+        return comparison_to_concept(parent, name, namespace, environment, metadata)
     else:
         return constant_to_concept(parent, name, namespace, metadata)
