@@ -62,6 +62,8 @@ from trilogy.core.models.author import (
     Conditional,
     CustomFunctionFactory,
     CustomType,
+    DeriveClause,
+    DeriveItem,
     Expr,
     FilterItem,
     Function,
@@ -69,6 +71,7 @@ from trilogy.core.models.author import (
     Grain,
     HavingClause,
     Metadata,
+    MultiSelectLineage,
     OrderBy,
     OrderItem,
     Parenthetical,
@@ -135,6 +138,7 @@ from trilogy.parsing.common import (
     align_item_to_concept,
     arbitrary_to_concept,
     constant_to_concept,
+    derive_item_to_concept,
     process_function_args,
     rowset_to_concepts,
 )
@@ -167,6 +171,16 @@ class FunctionBindingType:
     type: DataType | TraitDataType | None = None
 
 
+@dataclass
+class DropOn:
+    functions: List[FunctionType]
+
+
+@dataclass
+class AddOn:
+    functions: List[FunctionType]
+
+
 with open(join(dirname(__file__), "trilogy.lark"), "r") as f:
     PARSER = Lark(
         f.read(),
@@ -223,7 +237,14 @@ def expr_to_boolean(
 def unwrap_transformation(
     input: Expr,
     environment: Environment,
-) -> Function | FilterItem | WindowItem | AggregateWrapper | FunctionCallWrapper:
+) -> (
+    Function
+    | FilterItem
+    | WindowItem
+    | AggregateWrapper
+    | FunctionCallWrapper
+    | Parenthetical
+):
     if isinstance(input, Function):
         return input
     elif isinstance(input, AggregateWrapper):
@@ -243,7 +264,7 @@ def unwrap_transformation(
     elif isinstance(input, FunctionCallWrapper):
         return input
     elif isinstance(input, Parenthetical):
-        return unwrap_transformation(input.content, environment)
+        return input
     else:
         return Function.model_construct(
             operator=FunctionType.CONSTANT,
@@ -414,6 +435,9 @@ class ParseToObjects(Transformer):
     def IDENTIFIER(self, args) -> str:
         return args.value
 
+    def ORDER_IDENTIFIER(self, args) -> ConceptRef:
+        return self.environment.concepts[args.value.strip()].reference
+
     def WILDCARD_IDENTIFIER(self, args) -> str:
         return args.value
 
@@ -596,6 +620,9 @@ class ParseToObjects(Transformer):
     def PROPERTY(self, args):
         return Purpose.PROPERTY
 
+    def HASH_TYPE(self, args):
+        return args.value
+
     @v_args(meta=True)
     def prop_ident(self, meta: Meta, args) -> Tuple[List[Concept], str]:
         return [self.environment.concepts[grain] for grain in args[:-1]], args[-1]
@@ -700,7 +727,14 @@ class ParseToObjects(Transformer):
 
         if isinstance(
             source_value,
-            (FilterItem, WindowItem, AggregateWrapper, Function, FunctionCallWrapper),
+            (
+                FilterItem,
+                WindowItem,
+                AggregateWrapper,
+                Function,
+                FunctionCallWrapper,
+                Comparison,
+            ),
         ):
             concept = arbitrary_to_concept(
                 source_value,
@@ -779,7 +813,6 @@ class ParseToObjects(Transformer):
         lookup, namespace, name, parent = parse_concept_reference(
             name, self.environment
         )
-
         concept = Concept(
             name=name,
             datatype=arg_to_datatype(constant),
@@ -1269,6 +1302,17 @@ class ParseToObjects(Transformer):
     def align_clause(self, meta: Meta, args) -> AlignClause:
         return AlignClause(items=args)
 
+    @v_args(meta=True)
+    def derive_item(self, meta: Meta, args) -> DeriveItem:
+        return DeriveItem(
+            expr=args[0], name=args[1], namespace=self.environment.namespace
+        )
+
+    @v_args(meta=True)
+    def derive_clause(self, meta: Meta, args) -> DeriveClause:
+
+        return DeriveClause(items=args)
+
     @v_args(meta=True)
     def multi_select_statement(self, meta: Meta, args) -> MultiSelectStatement:
 
@@ -1278,6 +1322,7 @@ class ParseToObjects(Transformer):
         order_by: OrderBy | None = None
         where: WhereClause | None = None
         having: HavingClause | None = None
+        derive: DeriveClause | None = None
         for arg in args:
             if isinstance(arg, SelectStatement):
                 selects.append(arg)
@@ -1291,11 +1336,24 @@ class ParseToObjects(Transformer):
                 having = arg
             elif isinstance(arg, AlignClause):
                 align = arg
+            elif isinstance(arg, DeriveClause):
+                derive = arg
 
         assert align
         assert align is not None
 
         derived_concepts = []
+        new_selects = [x.as_lineage(self.environment) for x in selects]
+        lineage = MultiSelectLineage(
+            selects=new_selects,
+            align=align,
+            derive=derive,
+            namespace=self.environment.namespace,
+            where_clause=where,
+            having_clause=having,
+            limit=limit,
+            hidden_components=set(y for x in new_selects for y in x.hidden_components),
+        )
         for x in align.items:
             concept = align_item_to_concept(
                 x,
@@ -1308,6 +1366,19 @@ class ParseToObjects(Transformer):
             )
             derived_concepts.append(concept)
             self.environment.add_concept(concept, meta=meta)
+        if derive:
+            for derived in derive.items:
+                derivation = derived.expr
+                name = derived.name
+                if not isinstance(derivation, (Function, Comparison, WindowItem)):
+                    raise SyntaxError(
+                        f"Invalid derive expression {derivation} in {meta.line}, must be a function or conditional"
+                    )
+                concept = derive_item_to_concept(
+                    derivation, name, lineage, self.environment.namespace
+                )
+                derived_concepts.append(concept)
+                self.environment.add_concept(concept, meta=meta)
         multi = MultiSelectStatement(
             selects=selects,
             align=align,
@@ -1317,6 +1388,7 @@ class ParseToObjects(Transformer):
             limit=limit,
             meta=Metadata(line_number=meta.line),
             derived_concepts=derived_concepts,
+            derive=derive,
         )
         return multi
 
@@ -1335,6 +1407,10 @@ class ParseToObjects(Transformer):
             elif isinstance(arg, OrderBy):
                 order_by = arg
             elif isinstance(arg, WhereClause) and not isinstance(arg, HavingClause):
+                if where is not None:
+                    raise ParseError(
+                        "Multiple where clauses defined are not supported!"
+                    )
                 where = arg
             elif isinstance(arg, HavingClause):
                 having = arg
@@ -1444,17 +1520,42 @@ class ParseToObjects(Transformer):
         remapped = FunctionCallWrapper(
             content=self.environment.functions[name](*args), name=name, args=args
         )
+
         return remapped
 
     @v_args(meta=True)
     def function(self, meta: Meta, args) -> Function:
         return args[0]
 
+    @v_args(meta=True)
+    def type_drop_clause(self, meta: Meta, args) -> DropOn:
+        return DropOn([FunctionType(x) for x in args])
+
+    @v_args(meta=True)
+    def type_add_clause(self, meta: Meta, args) -> AddOn:
+        return AddOn([FunctionType(x) for x in args])
+
     @v_args(meta=True)
     def type_declaration(self, meta: Meta, args) -> TypeDeclaration:
         key = args[0]
-        datatype = args[1]
-        new = CustomType(name=key, type=datatype)
+        datatype: list[DataType] = [x for x in args[1:] if isinstance(x, DataType)]
+        if len(datatype) == 1:
+            final_datatype: list[DataType] | DataType = datatype[0]
+        else:
+            final_datatype = datatype
+        add_on = None
+        drop_on = None
+        for x in args[1:]:
+            if isinstance(x, AddOn):
+                add_on = x
+            elif isinstance(x, DropOn):
+                drop_on = x
+        new = CustomType(
+            name=key,
+            type=final_datatype,
+            drop_on=drop_on.functions if drop_on else [],
+            add_on=add_on.functions if add_on else [],
+        )
         self.environment.data_types[key] = new
         return TypeDeclaration(type=new)
 
@@ -1508,6 +1609,10 @@ class ParseToObjects(Transformer):
                 new_result = self.function_factory.create_function(
                     [result, right], operator=FunctionType.MULTIPLY
                 )
+            elif op == "**":
+                new_result = self.function_factory.create_function(
+                    [result, right], operator=FunctionType.POWER
+                )
             elif op == "/":
                 new_result = self.function_factory.create_function(
                     [result, right], operator=FunctionType.DIVIDE
@@ -1525,7 +1630,7 @@ class ParseToObjects(Transformer):
         return args.value
 
     def MULTIPLY_DIVIDE_PERCENT(self, args) -> str:
-        return args[0]
+        return args.value
 
     @v_args(meta=True)
     def sum_operator(self, meta: Meta, args) -> Function | Any:
@@ -1813,6 +1918,10 @@ class ParseToObjects(Transformer):
     def array_agg(self, meta, args):
         return self.function_factory.create_function(args, FunctionType.ARRAY_AGG, meta)
 
+    @v_args(meta=True)
+    def any(self, meta, args):
+        return self.function_factory.create_function(args, FunctionType.ANY, meta)
+
     @v_args(meta=True)
     def avg(self, meta, args):
         return self.function_factory.create_function(args, FunctionType.AVG, meta)
@@ -1873,6 +1982,10 @@ class ParseToObjects(Transformer):
     def ftrim(self, meta, args):
         return self.function_factory.create_function(args, FunctionType.TRIM, meta)
 
+    @v_args(meta=True)
+    def fhash(self, meta, args):
+        return self.function_factory.create_function(args, FunctionType.HASH, meta)
+
     @v_args(meta=True)
     def fsubstring(self, meta, args):
         return self.function_factory.create_function(args, FunctionType.SUBSTRING, meta)
@@ -1953,6 +2066,10 @@ class ParseToObjects(Transformer):
     def fday(self, meta, args):
         return self.function_factory.create_function(args, FunctionType.DAY, meta)
 
+    @v_args(meta=True)
+    def fday_name(self, meta, args):
+        return self.function_factory.create_function(args, FunctionType.DAY_NAME, meta)
+
     @v_args(meta=True)
     def fday_of_week(self, meta, args):
         return self.function_factory.create_function(
@@ -1967,6 +2084,12 @@ class ParseToObjects(Transformer):
     def fmonth(self, meta, args):
         return self.function_factory.create_function(args, FunctionType.MONTH, meta)
 
+    @v_args(meta=True)
+    def fmonth_name(self, meta, args):
+        return self.function_factory.create_function(
+            args, FunctionType.MONTH_NAME, meta
+        )
+
     @v_args(meta=True)
     def fquarter(self, meta, args):
         return self.function_factory.create_function(args, FunctionType.QUARTER, meta)
@@ -2006,6 +2129,12 @@ class ParseToObjects(Transformer):
             )
         return self.function_factory.create_function(args, FunctionType.CAST, meta)
 
+    @v_args(meta=True)
+    def fdate_spine(self, meta, args) -> Function:
+        return self.function_factory.create_function(
+            args, FunctionType.DATE_SPINE, meta
+        )
+
     # utility functions
     @v_args(meta=True)
     def fcast(self, meta, args) -> Function:
@@ -2175,6 +2304,33 @@ class ParseToObjects(Transformer):
             meta,
         )
 
+    @v_args(meta=True)
+    def farray_filter(self, meta, args) -> Function:
+        factory: CustomFunctionFactory = args[1]
+        if not len(factory.function_arguments) == 1:
+            raise InvalidSyntaxException(
+                "Array filter function must have exactly one argument;"
+            )
+        array_type = arg_to_datatype(args[0])
+        if not isinstance(array_type, ArrayType):
+            raise InvalidSyntaxException(
+                f"Array filter function must be applied to an array, not {array_type}"
+            )
+        return self.function_factory.create_function(
+            [
+                args[0],
+                factory.function_arguments[0],
+                factory(
+                    ArgBinding(
+                        name=factory.function_arguments[0].name,
+                        datatype=array_type.value_data_type,
+                    )
+                ),
+            ],
+            FunctionType.ARRAY_FILTER,
+            meta,
+        )
+
 
 def unpack_visit_error(e: VisitError, text: str | None = None):
     """This is required to get exceptions from imports, which would
@@ -2211,6 +2367,7 @@ ERROR_CODES: dict[int, str] = {
     101: "Using FROM keyword? Trilogy does not have a FROM clause (Datasource resolution is automatic).",
     # 200 codes relate to required explicit syntax (we could loosen these?)
     201: 'Missing alias? Alias must be specified with "AS" - e.g. `SELECT x+1 AS y`',
+    202: "Missing closing semicolon? Statements must be terminated with a semicolon `;`.",
     210: "Missing order direction? Order by must be explicit about direction - specify `asc` or `desc`.",
 }
 
@@ -2235,7 +2392,7 @@ def inject_context_maker(pos: int, text: str, span: int = 40) -> str:
         rcap = ""
         # if it goes beyond the end of text, no ...
         # if it terminates on a space, no need for ...
-        if not after[-1].isspace() and not (end > len(text)):
+        if after and not after[-1].isspace() and not (end > len(text)):
             rcap = "..."
         lcap = ""
         if start > 0 and not before[0].isspace():
@@ -2291,9 +2448,13 @@ def parse_text(
         )
 
     def _handle_unexpected_token(e: UnexpectedToken, text: str) -> None:
-        """Handle UnexpectedToken errors with specific logic."""
+        """Handle UnexpectedToken errors to make friendlier error messages."""
         # Handle ordering direction error
         pos = e.pos_in_stream or 0
+        if e.interactive_parser.lexer_thread.state:
+            last_token = e.interactive_parser.lexer_thread.state.last_token
+        else:
+            last_token = None
         if e.expected == {"ORDERING_DIRECTION"}:
             raise _create_syntax_error(210, pos, text)
 
@@ -2301,14 +2462,32 @@ def parse_text(
         parsed_tokens = (
             [x.value for x in e.token_history if x] if e.token_history else []
         )
+
         if parsed_tokens == ["FROM"]:
             raise _create_syntax_error(101, pos, text)
+        # check if they are missing a semicolon
+        if last_token and e.token.type == "$END":
+            try:
 
-        # Attempt recovery for aliasing
+                e.interactive_parser.feed_token(Token("_TERMINATOR", ";"))
+                state = e.interactive_parser.lexer_thread.state
+                if state and state.last_token:
+                    new_pos = state.last_token.end_pos or pos
+                else:
+                    new_pos = pos
+                raise _create_syntax_error(202, new_pos, text)
+            except UnexpectedToken:
+                pass
+        # check if they forgot an as
         try:
             e.interactive_parser.feed_token(Token("AS", "AS"))
+            state = e.interactive_parser.lexer_thread.state
+            if state and state.last_token:
+                new_pos = state.last_token.end_pos or pos
+            else:
+                new_pos = pos
             e.interactive_parser.feed_token(Token("IDENTIFIER", e.token.value))
-            raise _create_syntax_error(201, pos, text)
+            raise _create_syntax_error(201, new_pos, text)
         except UnexpectedToken:
             pass
 
