@@ -8,7 +8,14 @@ from typing import Any
 from jinja2 import Template
 
 from trilogy.constants import DEFAULT_NAMESPACE, VIRTUAL_CONCEPT_PREFIX, MagicConstants
-from trilogy.core.enums import ConceptSource, DatePart, FunctionType, Modifier, Purpose
+from trilogy.core.enums import (
+    ConceptSource,
+    DatePart,
+    FunctionType,
+    Modifier,
+    Purpose,
+    ValidationScope,
+)
 from trilogy.core.models.author import (
     AggregateWrapper,
     AlignClause,
@@ -66,6 +73,7 @@ from trilogy.core.statements.author import (
     SelectItem,
     SelectStatement,
     TypeDeclaration,
+    ValidateStatement,
 )
 
 QUERY_TEMPLATE = Template(
@@ -445,6 +453,13 @@ class Renderer:
         final = "".join(prefixes)
         return f"{final}{self.to_string(arg.content)}"
 
+    @to_string.register
+    def _(self, arg: ValidateStatement):
+        targets = ",".join(arg.targets) if arg.targets else "*"
+        if arg.scope.value == ValidationScope.ALL:
+            return "validate all;"
+        return f"validate {arg.scope.value} {targets};"
+
     @to_string.register
     def _(self, arg: SelectStatement):
         with self.indented():
@@ -557,8 +572,11 @@ class Renderer:
 
     @to_string.register
     def _(self, arg: "ConceptRef"):
+        if arg.address == "__preql_internal.all_rows":
+            return "*"
         if arg.name.startswith(VIRTUAL_CONCEPT_PREFIX) and self.environment:
             return self.to_string(self.environment.concepts[arg.address])
+
         ns, base = arg.address.rsplit(".", 1)
         if ns == DEFAULT_NAMESPACE:
             return base
@@ -625,7 +643,7 @@ class Renderer:
         if arg.operator == FunctionType.CONSTANT:
             return f"{', '.join(args)}"
         if arg.operator == FunctionType.CAST:
-            return f"CAST({self.to_string(arg.arguments[0])} AS {self.to_string(arg.arguments[1])})"
+            return f"{self.to_string(arg.arguments[0])}::{self.to_string(arg.arguments[1])}"
         if arg.operator == FunctionType.INDEX_ACCESS:
             return f"{self.to_string(arg.arguments[0])}[{self.to_string(arg.arguments[1])}]"
 
@@ -647,7 +665,8 @@ class Renderer:
                     pair_strings.append(self.indent_lines(pair_line))
             inputs = ",\n".join(pair_strings)
             return f"struct(\n{inputs}\n{self.indent_context.current_indent})"
-
+        if arg.operator == FunctionType.ALIAS:
+            return f"{self.to_string(arg.arguments[0])}"
         inputs = ",".join(args)
         return f"{arg.operator.value}({inputs})"
 
